var birthdayRegex = /\b(?:n\/a|\d{1,2}[\\/-]\d{1,2}[\\/-](?:\d{2}|\d{4})|\d{4}[\\/-]\d{1,2}[\\/-]\d{1,2}|(?:Jan(?:uary)?|Feb(?:ruary)?|Mar(?:ch)?|Apr(?:il)?|May|Jun(?:e)?|Jul(?:y)?|Aug(?:ust)?|Sep(?:tember)?|Oct(?:ober)?|Nov(?:ember)?|Dec(?:ember)?)\s*\d{0,2}(?:,?\s*(?:\d{2}|\d{4}))?)\b/i;
var emailRegex = /\b[A-Za-z0-9._%+-]+@[A-Za-z0-9.-]+\.[A-Za-z]{2,}\b/g;
var defaultPhoneRegex = /^[+]?[(]?[0-9]{3}[)]?[-\s.]?[0-9]{3}[-\s.]?[0-9]{2,6}$/;
var defaultInputAttrs = [
    "autocomplete",
    "placeholder",
    "name",
    "id",
    "class",
];
var AutoCapturePII = /** @class */ (function () {
    function AutoCapturePII(browser, user, phoneRegex) {
        this.user = user;
        this.browser = browser;
        this.phoneRegex = phoneRegex ? new RegExp(phoneRegex) : defaultPhoneRegex;
        this.boundOnChange = this.onChange.bind(this);
    }
    AutoCapturePII.prototype.listen = function () {
        var _this = this;
        var _a, _b, _c;
        (_c = (_b = (_a = this.browser) === null || _a === void 0 ? void 0 : _a.document()) === null || _b === void 0 ? void 0 : _b.body) === null || _c === void 0 ? void 0 : _c.addEventListener("change", this.boundOnChange, {
            capture: true,
        });
        return function () {
            var _a, _b, _c;
            (_c = (_b = (_a = _this.browser) === null || _a === void 0 ? void 0 : _a.document()) === null || _b === void 0 ? void 0 : _b.body) === null || _c === void 0 ? void 0 : _c.removeEventListener("change", _this.boundOnChange, {
                capture: true,
            });
        };
    };
    AutoCapturePII.prototype.onChange = function (e) {
        var _this = this;
        var traits = {};
        // We only want to capture PII from input and select elements
        var possibleTargets = [HTMLInputElement, HTMLSelectElement];
        if (!possibleTargets.some(function (t) { return e.target instanceof t; })) {
            return;
        }
        var inputTarget = e.target;
        var selectTarget = e.target;
        var type = (inputTarget === null || inputTarget === void 0 ? void 0 : inputTarget.type) || (selectTarget === null || selectTarget === void 0 ? void 0 : selectTarget.type);
        // Normalize the value by removing spaces and converting to lowercase
        var split = inputTarget.value.trim().split(" ");
        var value = split.join("").trim().toLocaleLowerCase();
        switch (type) {
            case "text":
                if (this.checkInputBy(inputTarget, ["firstname"])) {
                    traits.firstname = inputTarget.value.trim();
                    break;
                }
                if (this.checkInputBy(inputTarget, ["lastname"])) {
                    traits.lastname = inputTarget.value.trim();
                    break;
                }
                if (this.checkInputBy(inputTarget, ["name"])) {
                    traits.name = inputTarget.value.trim();
                    if (split.length > 1) {
                        traits.firstname = split[0].trim();
                        traits.lastname = split.slice(1).join(" ").trim();
                    }
                    break;
                }
                if (this.checkInputBy(inputTarget, ["mobile", "tel"], this.phoneRegex) &&
                    this.isValidPhone(value)) {
                    traits.phone = value;
                    break;
                }
                if (this.checkInputBy(inputTarget, ["email", "e-mail"], emailRegex) &&
                    this.isValidEmail(value)) {
                    traits.email = value;
                }
                break;
            case "email":
                if (this.isValidEmail(value)) {
                    traits.email = value;
                }
                break;
            case "tel":
            case "phone":
            case "mobile":
            case "number":
                if (this.isValidPhone(value)) {
                    traits.phone = value;
                }
                break;
            case "radio":
            case "checkbox":
                if (this.getGender(value) && (inputTarget === null || inputTarget === void 0 ? void 0 : inputTarget.checked)) {
                    traits.gender = this.getGender(value);
                }
                break;
            case "date":
                if (this.isValidBirthday(inputTarget)) {
                    traits.birthday = value;
                }
                break;
            case "select-one":
                this.getSelectValues(selectTarget).forEach(function (v) {
                    var g = _this.getGender(v);
                    if (g) {
                        traits.gender = g;
                    }
                });
                break;
        }
        if (Object.keys(traits).length > 0) {
            this.user.setTraits(traits);
        }
    };
    // Check if the input element has a specific attribute or value that matches the keys
    // If a regex is provided, it will also check if the value matches the regex
    AutoCapturePII.prototype.checkInputBy = function (e, keys, regex) {
        var _a;
        for (var _i = 0, keys_1 = keys; _i < keys_1.length; _i++) {
            var key = keys_1[_i];
            for (var _b = 0, defaultInputAttrs_1 = defaultInputAttrs; _b < defaultInputAttrs_1.length; _b++) {
                var attr = defaultInputAttrs_1[_b];
                var value = (_a = e.getAttribute(attr)) === null || _a === void 0 ? void 0 : _a.toLowerCase();
                if ((regex && (value === null || value === void 0 ? void 0 : value.match(regex))) || (value === null || value === void 0 ? void 0 : value.includes(key))) {
                    return true;
                }
            }
        }
        return false;
    };
    AutoCapturePII.prototype.isValidEmail = function (v) {
        return emailRegex.test(v);
    };
    AutoCapturePII.prototype.isValidPhone = function (v) {
        return this.phoneRegex.test(v);
    };
    AutoCapturePII.prototype.isValidBirthday = function (e) {
        return birthdayRegex.test(e === null || e === void 0 ? void 0 : e.value);
    };
    AutoCapturePII.prototype.getGender = function (value) {
        var possibleValues = ["m", "male", "f", "female"];
        if (possibleValues.find(function (v) { return v === (value === null || value === void 0 ? void 0 : value.toLocaleLowerCase()); })) {
            return value[0].toLocaleLowerCase(); // return the first letter
        }
        return null;
    };
    // Returns an array with the value and the label of the selected option
    // ex <option value="1" label="male">male</option>
    AutoCapturePII.prototype.getSelectValues = function (e) {
        var _a;
        if ((e === null || e === void 0 ? void 0 : e.selectedIndex) === -1)
            return [];
        return [e === null || e === void 0 ? void 0 : e.value, (_a = e === null || e === void 0 ? void 0 : e.options[e === null || e === void 0 ? void 0 : e.selectedIndex]) === null || _a === void 0 ? void 0 : _a.label];
    };
    return AutoCapturePII;
}());
export { AutoCapturePII };
//# sourceMappingURL=autoCapturePII.js.map