var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import { EmitterImpl } from "../transport/emitter";
var OperationsPriorityQueueImpl = /** @class */ (function (_super) {
    __extends(OperationsPriorityQueueImpl, _super);
    function OperationsPriorityQueueImpl(maxAttempts) {
        var _this = _super.call(this) || this;
        _this.maxAttempts = maxAttempts;
        _this.nowOperations = [];
        _this.delayedOperations = [];
        _this.attempts = {};
        return _this;
    }
    OperationsPriorityQueueImpl.prototype.push = function () {
        var _this = this;
        var operations = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            operations[_i] = arguments[_i];
        }
        var accepted = operations.map(function (operation) {
            var attempts = _this.newAttempt(operation);
            if (attempts > _this.maxAttempts || _this.alreadyInQueue(operation)) {
                return false;
            }
            _this.nowOperations.push(operation);
            return true;
        });
        this.nowOperations = this.nowOperations.sort(function (a, b) { return _this.getAttempts(a) - _this.getAttempts(b); });
        return accepted;
    };
    OperationsPriorityQueueImpl.prototype.pushWithBackoff = function (operation) {
        var _this = this;
        if (this.getAttempts(operation) === 0) {
            return this.push(operation)[0];
        }
        var attempts = this.newAttempt(operation);
        if (attempts > this.maxAttempts || this.alreadyInQueue(operation)) {
            return false;
        }
        this.delayedOperations.push(operation);
        var delayMs = this.backoffDelayInMs(attempts - 1);
        setTimeout(function () {
            _this.nowOperations.push(operation);
            _this.delayedOperations = _this.delayedOperations.filter(function (f) { return f.getId() !== operation.getId(); });
            _this.emit(ON_OPERATION_DELAY_FINISH);
        }, delayMs);
        return true;
    };
    OperationsPriorityQueueImpl.prototype.isEmpty = function () {
        return this.nowOperations.length === 0;
    };
    OperationsPriorityQueueImpl.prototype.pop = function () {
        return this.nowOperations.shift();
    };
    OperationsPriorityQueueImpl.prototype.backoffDelayInMs = function (attempt) {
        var random = Math.random() + 1;
        var minTimeout = 500;
        var factor = 2;
        return random * minTimeout * Math.pow(factor, attempt);
    };
    OperationsPriorityQueueImpl.prototype.newAttempt = function (operation) {
        this.attempts[operation.getId()] = this.getAttempts(operation) + 1;
        return this.getAttempts(operation);
    };
    OperationsPriorityQueueImpl.prototype.getAttempts = function (operation) {
        var _a;
        return (_a = this.attempts[operation.getId()]) !== null && _a !== void 0 ? _a : 0;
    };
    OperationsPriorityQueueImpl.prototype.alreadyInQueue = function (operation) {
        return (this.nowOperations.includes(operation) ||
            this.delayedOperations.includes(operation) ||
            Boolean(this.nowOperations.find(function (i) { return i.getId() === operation.getId(); })) ||
            Boolean(this.delayedOperations.find(function (i) { return i.getId() === operation.getId(); })));
    };
    return OperationsPriorityQueueImpl;
}(EmitterImpl));
export { OperationsPriorityQueueImpl };
export var ON_OPERATION_DELAY_FINISH = "onOperationDelayFinish";
//# sourceMappingURL=priorityQueue.js.map