import * as Sentry from "@sentry/browser";
import { LIB_VERSION } from "../generated/libVersion";
var isLocalHost = window.location.hostname === "localhost" ||
    window.location.hostname === "lvh.me";
var SentryWrapperImpl = /** @class */ (function () {
    function SentryWrapperImpl() {
        var integrations = Sentry.getDefaultIntegrations({})
            .filter(function (defaultIntegration) {
            return ![
                "BrowserApiErrors",
                "TryCatch",
                "Breadcrumbs",
                "GlobalHandlers",
            ].includes(defaultIntegration.name);
        })
            .concat([Sentry.browserProfilingIntegration()]);
        var client = new Sentry.BrowserClient({
            enabled: !isLocalHost,
            release: LIB_VERSION,
            dsn: "https://11576b3befe62387b6f5ad58d9bd738f@o4506150022152192.ingest.us.sentry.io/4507033659375616",
            sampleRate: 0.001,
            tracesSampleRate: 0.001,
            replaysSessionSampleRate: 0,
            replaysOnErrorSampleRate: 0.2,
            transport: Sentry.makeFetchTransport,
            stackParser: Sentry.defaultStackParser,
            integrations: integrations,
            ignoreErrors: [
                "SecurityError",
                "the user id is required when calling identify",
                "TypeError: Failed to fetch",
                "TypeError: Load failed",
                "TypeError: NetworkError when attempting to fetch resource.",
            ],
        });
        this.scope = new Sentry.Scope();
        this.scope.setClient(client);
        this.scope.setTag("domain", window.location.hostname);
        client.init();
    }
    SentryWrapperImpl.prototype.captureMessage = function (message, level, hint) {
        return this.scope.captureMessage(message, level, hint);
    };
    SentryWrapperImpl.prototype.captureException = function (error, hint) {
        return this.scope.captureException(error, hint);
    };
    SentryWrapperImpl.prototype.setResponse = function (response) {
        if (!response) {
            return;
        }
        this.scope.setExtra("response", response);
    };
    SentryWrapperImpl.prototype.setTag = function (key, value) {
        this.scope.setTag(key, value);
    };
    return SentryWrapperImpl;
}());
export { SentryWrapperImpl };
//# sourceMappingURL=sentry.js.map