import { MemoryStore } from "./memoryStore";
var BrowserStore = /** @class */ (function () {
    function BrowserStore(storage) {
        this.browserStorage = null;
        this.memoryStore = null;
        if (storage) {
            this.browserStorage = storage;
        }
        else {
            this.memoryStore = new MemoryStore();
        }
    }
    BrowserStore.isLocalStorageAvailable = function () {
        var testKey = "journify.io-test-browser-storage-key";
        try {
            localStorage.setItem(testKey, "journify.io-test-browser-storage-value");
            localStorage.removeItem(testKey);
            return true;
        }
        catch (e) {
            return false;
        }
    };
    BrowserStore.prototype.get = function (key) {
        var val = this.browserStorage
            ? this.browserStorage.getItem(key)
            : this.memoryStore.get(key);
        if (val) {
            try {
                return JSON.parse(val);
            }
            catch (e) {
                return JSON.parse(JSON.stringify(val));
            }
        }
        return null;
    };
    BrowserStore.prototype.set = function (key, value) {
        try {
            var persistedValue = void 0;
            switch (typeof value) {
                case "string":
                case "number":
                case "boolean":
                    persistedValue = value.toString();
                    break;
                default:
                    persistedValue = JSON.stringify(value);
                    break;
            }
            this.browserStorage
                ? this.browserStorage.setItem(key, persistedValue)
                : this.memoryStore.set(key, persistedValue);
        }
        catch (e) {
            console.warn("Unable to set ".concat(key, " in browser storage, storage may be full."));
            console.warn(e);
            return null;
        }
        return value;
    };
    BrowserStore.prototype.remove = function (key) {
        try {
            this.browserStorage
                ? this.browserStorage.removeItem(key)
                : this.memoryStore.remove(key);
        }
        catch (e) {
            console.warn("Unable to remove ".concat(key, " from browser storage."));
            console.warn(e);
        }
    };
    return BrowserStore;
}());
export { BrowserStore };
//# sourceMappingURL=browserStore.js.map