import JsCookie from "js-cookie";
var ONE_YEAR = 365;
var CookiesStore = /** @class */ (function () {
    function CookiesStore(domain) {
        this.domain = domain;
    }
    CookiesStore.isAvailable = function () {
        var cookieEnabled = navigator === null || navigator === void 0 ? void 0 : navigator.cookieEnabled;
        if (!cookieEnabled) {
            var testKey = "journify.io-test-cookie-key";
            JsCookie.set(testKey, "journify.io-test-cookie-value");
            cookieEnabled = document.cookie.includes(testKey);
            JsCookie.remove(testKey);
        }
        return cookieEnabled;
    };
    CookiesStore.prototype.get = function (key) {
        try {
            var value = JsCookie.get(key);
            if (!value) {
                return null;
            }
            try {
                return JSON.parse(value);
            }
            catch (e) {
                return value;
            }
        }
        catch (e) {
            return null;
        }
    };
    CookiesStore.prototype.set = function (key, value) {
        var attributes = this.getAttributes();
        if (value === null || value === undefined || value === "") {
            JsCookie.remove(key, attributes);
            return null;
        }
        var parsedValue = "";
        switch (typeof value) {
            case "string":
                parsedValue = value;
                break;
            case "number":
            case "boolean":
                parsedValue = value.toString();
                break;
            default:
                parsedValue = JSON.stringify(value);
        }
        JsCookie.set(key, parsedValue, attributes);
        return value;
    };
    CookiesStore.prototype.remove = function (key) {
        return JsCookie.remove(key, this.getAttributes());
    };
    CookiesStore.prototype.getAttributes = function () {
        if (!this.attributes) {
            this.setAttributes();
        }
        return this.attributes;
    };
    CookiesStore.prototype.setAttributes = function () {
        this.attributes = {
            expires: ONE_YEAR,
            path: "/",
            domain: this.getDomainAttribute(),
            sameSite: "Lax",
        };
    };
    CookiesStore.prototype.getDomainAttribute = function () {
        if (this.domain) {
            return this.domain;
        }
        var parsedUrl = parseUrl(location === null || location === void 0 ? void 0 : location.href);
        if (!parsedUrl)
            return;
        var levels = getUrlLevels(parsedUrl);
        // Lookup the real top level one.
        for (var i = 0; i < levels.length; ++i) {
            var cname = "__tld__";
            var domain = levels[i];
            var opts = { domain: "." + domain };
            try {
                // cookie access throw an error if the library is run inside a sandboxed environment (e.g. sandboxed iframe)
                JsCookie.set(cname, "1", opts);
                if (JsCookie.get(cname)) {
                    JsCookie.remove(cname, opts);
                    return domain;
                }
            }
            catch (_) {
                return;
            }
        }
    };
    return CookiesStore;
}());
export { CookiesStore };
function parseUrl(url) {
    try {
        return new URL(url);
    }
    catch (_a) {
        return;
    }
}
function getUrlLevels(url) {
    var host = url.hostname;
    var parts = host.split(".");
    var last = parts[parts.length - 1];
    var levels = [];
    // Ip address.
    if (parts.length === 4 && parseInt(last, 10) > 0) {
        return levels;
    }
    // Localhost.
    if (parts.length <= 1) {
        return levels;
    }
    // Create levels.
    for (var i = parts.length - 2; i >= 0; --i) {
        levels.push(parts.slice(i).join("."));
    }
    return levels;
}
//# sourceMappingURL=cookiesStore.js.map