import { MemoryStore } from "./memoryStore";
var SessionStore = /** @class */ (function () {
    function SessionStore(browser) {
        this.sessionStorage = null;
        this.memoryStore = null;
        if (this.isAvailable(browser)) {
            this.sessionStorage = browser.window().sessionStorage;
        }
        else {
            this.memoryStore = new MemoryStore();
        }
    }
    SessionStore.prototype.isAvailable = function (browser) {
        var testKey = "journify.io-test-browser-storage-key";
        try {
            browser
                .window()
                .sessionStorage.setItem(testKey, "journify.io-test-browser-storage-value");
            browser.window().sessionStorage.removeItem(testKey);
            return true;
        }
        catch (e) {
            return false;
        }
    };
    SessionStore.prototype.get = function (key) {
        var val = this.sessionStorage
            ? this.sessionStorage.getItem(key)
            : this.memoryStore.get(key);
        if (val) {
            try {
                return JSON.parse(val);
            }
            catch (e) {
                return JSON.parse(JSON.stringify(val));
            }
        }
        return null;
    };
    SessionStore.prototype.set = function (key, value) {
        try {
            var stringValue = JSON.stringify(value);
            this.sessionStorage
                ? this.sessionStorage.setItem(key, stringValue)
                : this.memoryStore.set(key, stringValue);
        }
        catch (e) {
            console.warn("Unable to set ".concat(key, " in session storage, storage may be full."));
            console.warn(e);
            return null;
        }
        return value;
    };
    SessionStore.prototype.remove = function (key) {
        try {
            this.sessionStorage
                ? this.sessionStorage.removeItem(key)
                : this.memoryStore.remove(key);
        }
        catch (e) {
            console.warn("Unable to remove ".concat(key, " from session storage."));
            console.warn(e);
        }
    };
    return SessionStore;
}());
export { SessionStore };
//# sourceMappingURL=sessionStore.js.map