var BrowserImpl = /** @class */ (function () {
    function BrowserImpl() {
    }
    BrowserImpl.prototype.injectScript = function (url, opts) {
        var _a;
        var script = this.document().createElement("script");
        script.src = url;
        script.async = opts.async;
        script.onload = opts.onload;
        var doc = this.document();
        var scripts = doc.getElementsByTagName("script");
        if (scripts.length > 0) {
            (_a = scripts[0].parentNode) === null || _a === void 0 ? void 0 : _a.insertBefore(script, null);
        }
        else {
            doc.head.appendChild(script);
        }
    };
    BrowserImpl.prototype.scriptAlreadyInPage = function (scriptUrl) {
        return Array.from(this.document().scripts).some(function (script) {
            return script.src.includes(scriptUrl);
        });
    };
    BrowserImpl.prototype.isCurrentPageHttps = function () {
        var _a;
        return ((_a = this.location()) === null || _a === void 0 ? void 0 : _a.protocol) === "https:";
    };
    BrowserImpl.prototype.navigator = function () {
        return navigator;
    };
    BrowserImpl.prototype.document = function () {
        return document;
    };
    BrowserImpl.prototype.location = function () {
        return location;
    };
    BrowserImpl.prototype.window = function () {
        return window;
    };
    BrowserImpl.prototype.isOnline = function () {
        var _a;
        return ((_a = this.navigator()) === null || _a === void 0 ? void 0 : _a.onLine) || false;
    };
    BrowserImpl.prototype.canonicalPath = function () {
        var _a;
        var canon = this.canonical();
        if (!canon) {
            return (_a = this.location()) === null || _a === void 0 ? void 0 : _a.pathname;
        }
        var a = this.document().createElement("a");
        a.href = canon;
        return !a.pathname.startsWith("/") ? "/" + a.pathname : a.pathname;
    };
    BrowserImpl.prototype.canonicalUrl = function () {
        var _a, _b;
        var canon = this.canonical();
        if (canon) {
            return canon.includes("?") ? canon : "".concat(canon).concat((_a = this.location()) === null || _a === void 0 ? void 0 : _a.search);
        }
        var url = (_b = this.location()) === null || _b === void 0 ? void 0 : _b.href;
        var i = url.indexOf("#");
        return i === -1 ? url : url.slice(0, i);
    };
    BrowserImpl.prototype.utmCampaign = function (queryString, store) {
        var _this = this;
        var utm = {};
        var sParams = new URLSearchParams(queryString);
        var campaignFound = false;
        UTM_KEYS.forEach(function (k) {
            var paramName = k[0];
            var param = _this.getUtmParam(paramName, sParams, store);
            if (param) {
                var fieldName = k[1];
                campaignFound = true;
                utm[fieldName] = param;
            }
        });
        return campaignFound ? utm : undefined;
    };
    BrowserImpl.prototype.canonical = function () {
        var tag = this.document().querySelector("link[rel='canonical']");
        if (!tag) {
            return null;
        }
        return tag.getAttribute("href");
    };
    BrowserImpl.prototype.getUtmParam = function (paramName, sParams, store) {
        var value = sParams.get(paramName) || store.get(paramName);
        if (!value) {
            return null;
        }
        store.set(paramName, value);
        return value.toString();
    };
    return BrowserImpl;
}());
export { BrowserImpl };
export var UTM_KEYS = [
    ["utm_id", "id"],
    ["utm_campaign", "name"],
    ["utm_source", "source"],
    ["utm_medium", "medium"],
    ["utm_term", "term"],
    ["utm_content", "content"],
];
//# sourceMappingURL=browser.js.map