var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
var CleverTapWrapperImpl = /** @class */ (function () {
    function CleverTapWrapperImpl(browser, logger, testingWriteKey) {
        this.scriptInitialized = false;
        this.browser = browser;
        this.logger = logger;
        this.testingMode = testingWriteKey;
    }
    CleverTapWrapperImpl.prototype.init = function (accountId, region) {
        this.scriptInitialized = false;
        this.initScript();
        if (this.testingMode) {
            this.logger.log("clevertap.account.push is called with: accountID: ".concat(accountId, ", region: ").concat(region));
        }
        else {
            var clevertap = this.getCleverTapObject();
            clevertap.account.push({ id: accountId }, region);
        }
    };
    CleverTapWrapperImpl.prototype.pushProfileData = function (profileData) {
        this.initScript();
        if (this.testingMode) {
            this.logger.log("Will call clevertap.onUserLogin.push with the following param:", profileData);
            return;
        }
        var clevertap = this.getCleverTapObject();
        clevertap.onUserLogin.push(profileData);
    };
    CleverTapWrapperImpl.prototype.pushEvent = function (evtName) {
        var _a;
        var evtNameOrData = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            evtNameOrData[_i - 1] = arguments[_i];
        }
        this.initScript();
        if (this.testingMode) {
            this.logger.log("Will call clevertap.event.push with the following params in order:", __spreadArray([evtName], evtNameOrData, true));
            return;
        }
        var clevertap = this.getCleverTapObject();
        (_a = clevertap.event).push.apply(_a, __spreadArray([evtName], evtNameOrData, false));
    };
    CleverTapWrapperImpl.prototype.initScript = function () {
        if (this.scriptInitialized) {
            return;
        }
        else if (this.testingMode) {
            this.logger.log("CleverTap is detected but script is not injected because you are using a testing write key.");
            this.scriptInitialized = true;
            return;
        }
        var win = this.browser.window();
        win.clevertap = {
            account: [],
            event: [],
            profile: [],
            onUserLogin: [],
            notifications: [],
            privacy: [],
        };
        var isHttps = this.browser.isCurrentPageHttps();
        var scriptHost = isHttps
            ? "https://d2r1yp2w7bby2u.cloudfront.net"
            : "http://static.clevertap.com";
        var scriptUrl = scriptHost + "/js/clevertap.min.js";
        this.browser.injectScript(scriptUrl, { async: true });
        this.scriptInitialized = true;
    };
    CleverTapWrapperImpl.prototype.getCleverTapObject = function () {
        return this.browser.window().clevertap;
    };
    return CleverTapWrapperImpl;
}());
export { CleverTapWrapperImpl };
//# sourceMappingURL=cleverTapWrapper.js.map