var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import { matchFilters } from "../lib/filters";
import { facebookBirthday, oneLetterGender, toDigitsOnlyPhone, toLowerCase, trim, } from "../lib/tranformations";
import { JournifyEventType } from "../../../domain/event";
import { toSettingsObject } from "../lib/settings";
import { getStoredIdentify } from "../lib/identify";
var STANDARD_EVENTS = new Set([
    "AddPaymentInfo",
    "AddToCart",
    "AddToWishlist",
    "CompleteRegistration",
    "Contact",
    "CustomizeProduct",
    "Donate",
    "FindLocation",
    "InitiateCheckout",
    "Lead",
    "Purchase",
    "PageView",
    "Schedule",
    "Search",
    "StartTrial",
    "SubmitApplication",
    "Subscribe",
    "ViewContent",
]);
var USER_DATA_FIELDS = new Set([
    "em",
    "fn",
    "ln",
    "ph",
    "external_id",
    "ge",
    "db",
    "ct",
    "st",
    "zp",
    "country",
]);
var FACEBOOK_SCRIPT_URL = "https://connect.facebook.net/en_US/fbevents.js";
var DEFAULT_CURRENCY = "USD";
var FacebookPixel = /** @class */ (function () {
    function FacebookPixel(deps) {
        this.name = "facebook";
        this.settings = {};
        this.user = deps.user;
        this.browser = deps.browser;
        this.fieldsMapper = deps.fieldMapperFactory.newFieldMapper(deps.sync.field_mappings);
        this.testingMode = deps.testingWriteKey;
        this.eventMapper = deps.eventMapperFactory.newEventMapper(deps.sync.event_mappings);
        this.logger = deps.logger;
        this.init(deps.sync);
    }
    FacebookPixel.prototype.track = function (ctx) {
        return this.trackPixelEvent(ctx);
    };
    FacebookPixel.prototype.page = function (ctx) {
        return this.trackPixelEvent(ctx);
    };
    FacebookPixel.prototype.group = function (ctx) {
        return this.trackPixelEvent(ctx);
    };
    FacebookPixel.prototype.identify = function (ctx) {
        var newEvent = ctx.getEvent();
        var storedEvent = getStoredIdentify(this.user);
        var event = {
            type: JournifyEventType.IDENTIFY,
            userId: newEvent.userId || storedEvent.userId,
            anonymousId: newEvent.anonymousId || storedEvent.anonymousId,
            traits: __assign(__assign({}, (storedEvent.traits || {})), (newEvent.traits || {})),
        };
        this.initPixel(event);
        return this.trackPixelEvent(ctx);
    };
    FacebookPixel.prototype.updateSettings = function (sync) {
        this.init(sync);
    };
    FacebookPixel.prototype.trackPixelEvent = function (ctx) {
        var event = ctx.getEvent();
        var eventMapping = this.eventMapper.getEventMapping(event);
        if (!eventMapping || !matchFilters(event, eventMapping === null || eventMapping === void 0 ? void 0 : eventMapping.filters)) {
            return ctx;
        }
        var mappedProperties = this.fieldsMapper.mapEvent(event);
        var eventId = mappedProperties.event_id;
        delete mappedProperties.event_id;
        var eventName = (eventMapping === null || eventMapping === void 0 ? void 0 : eventMapping.pixelEventName) || event.event;
        var trackType = isStandardEvent(eventName)
            ? "trackSingle"
            : "trackSingleCustom";
        var properties = this.removeUserData(mappedProperties);
        if (!properties.currency) {
            properties.currency = DEFAULT_CURRENCY;
        }
        this.callPixelHelper(trackType, this.settings.pixel_id, eventName, properties, { eventID: eventId });
        return ctx;
    };
    FacebookPixel.prototype.removeUserData = function (properties) {
        var output = {};
        for (var key in properties) {
            if (USER_DATA_FIELDS.has(key)) {
                continue;
            }
            output[key] = properties[key];
        }
        return output;
    };
    FacebookPixel.prototype.init = function (sync) {
        this.settings = toSettingsObject(sync.settings);
        if (this.testingMode) {
            this.logger.log("Facebook Pixel ".concat(this.settings.pixel_id, " is detected, but script is not injected because you are using a testing write key."));
        }
        else {
            this.loadFbScript();
        }
        var event = getStoredIdentify(this.user);
        this.initPixel(event);
    };
    FacebookPixel.prototype.loadFbScript = function () {
        var localWindow = this.browser.window();
        if (localWindow.fbq) {
            return;
        }
        var n = null;
        n = localWindow.fbq = function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            n.callMethod ? n.callMethod.apply(n, args) : n.queue.push(args);
        };
        if (!localWindow._fbq)
            localWindow._fbq = n;
        n.push = n;
        n.loaded = !0;
        n.version = "2.0";
        n.queue = [];
        this.browser.injectScript(FACEBOOK_SCRIPT_URL, { async: true });
    };
    FacebookPixel.prototype.initPixel = function (identifyEvent) {
        var userData = this.mapUserData(identifyEvent);
        this.callPixelHelper("init", this.settings.pixel_id, userData);
    };
    FacebookPixel.prototype.callPixelHelper = function () {
        var _a;
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        if (this.testingMode) {
            this.logger.log("Will call window.fbq with the following params in order:", args);
            return;
        }
        (_a = this.browser.window()).fbq.apply(_a, args);
    };
    FacebookPixel.prototype.mapUserData = function (event) {
        var transformationsMap = {
            em: [trim, toLowerCase],
            fn: [trim, toLowerCase],
            ln: [trim, toLowerCase],
            ph: [toDigitsOnlyPhone],
            city: [trim, toLowerCase],
            st: [trim, toLowerCase],
            country: [toLowerCase],
            ge: [oneLetterGender, toLowerCase],
            db: [facebookBirthday],
        };
        return this.fieldsMapper.mapEvent(event, transformationsMap);
    };
    return FacebookPixel;
}());
export { FacebookPixel };
function isStandardEvent(eventName) {
    return STANDARD_EVENTS.has(eventName);
}
//# sourceMappingURL=facebookPixel.js.map