var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import { toSettingsObject } from "../lib/settings";
import { getStoredIdentify } from "../lib/identify";
import { JournifyEventType } from "../../../domain/event";
import { toLowerCase, trim } from "../lib/tranformations";
import { matchFilters } from "../lib/filters";
var GoogleAdsGtag = /** @class */ (function () {
    function GoogleAdsGtag(deps) {
        this.name = "google_ads_gtag";
        this.settings = {};
        this.user = deps.user;
        this.browser = deps.browser;
        this.fieldsMapper = deps.fieldMapperFactory.newFieldMapper(deps.sync.field_mappings);
        this.testingMode = deps.testingWriteKey;
        this.eventMapper = deps.eventMapperFactory.newEventMapper(deps.sync.event_mappings);
        this.logger = deps.logger;
        this.init(deps.sync);
    }
    GoogleAdsGtag.prototype.track = function (ctx) {
        return this.trackGtagEvent(ctx);
    };
    GoogleAdsGtag.prototype.page = function (ctx) {
        return this.trackGtagEvent(ctx);
    };
    GoogleAdsGtag.prototype.group = function (ctx) {
        return this.trackGtagEvent(ctx);
    };
    GoogleAdsGtag.prototype.identify = function (ctx) {
        var newEvent = ctx.getEvent();
        var storedEvent = getStoredIdentify(this.user);
        var event = {
            type: JournifyEventType.IDENTIFY,
            userId: newEvent.userId || storedEvent.userId,
            anonymousId: newEvent.anonymousId || storedEvent.anonymousId,
            traits: __assign(__assign({}, (storedEvent.traits || {})), (newEvent.traits || {})),
        };
        this.initTag(event);
        return this.trackGtagEvent(ctx);
    };
    GoogleAdsGtag.prototype.updateSettings = function (sync) {
        this.init(sync);
    };
    GoogleAdsGtag.prototype.trackGtagEvent = function (ctx) {
        var event = ctx.getEvent();
        var eventMapping = this.eventMapper.getEventMapping(event);
        if (!eventMapping || !matchFilters(event, eventMapping === null || eventMapping === void 0 ? void 0 : eventMapping.filters)) {
            return ctx;
        }
        var _a = this.parseMappedEventName(eventMapping.pixelEventName), conversionID = _a.conversionID, conversionType = _a.conversionType;
        var mappedProperties = this.fieldsMapper.mapEvent(event);
        delete mappedProperties.conversionAction;
        var properties = this.removeUserInitData(mappedProperties);
        if (event.traits && Object.keys(event.traits).length > 0) {
            var storedIdentify = getStoredIdentify(this.user);
            var identifyEvent = {
                type: JournifyEventType.IDENTIFY,
                userId: event.userId || storedIdentify.userId,
                anonymousId: event.anonymousId || storedIdentify.anonymousId,
                traits: __assign(__assign({}, (storedIdentify.traits || {})), (event.traits || {})),
            };
            this.setUserData(identifyEvent);
        }
        switch (conversionType) {
            case "UPLOAD_CALLS":
                this.gtag("config", conversionID, properties);
                break;
            default:
                delete properties.phone_conversion_number;
                this.gtag("event", "conversion", __assign({ send_to: conversionID }, properties));
                break;
        }
        return ctx;
    };
    GoogleAdsGtag.prototype.parseMappedEventName = function (mappedEvent) {
        var split = mappedEvent.split("/");
        var conversionID = split.slice(0, split.length - 1).join("/");
        var conversionType = split[split.length - 1];
        return { conversionID: conversionID, conversionType: conversionType };
    };
    GoogleAdsGtag.prototype.init = function (sync) {
        this.settings = toSettingsObject(sync.settings);
        if (this.testingMode) {
            this.logger.log("Google Ads gtag ".concat(this.settings.google_ads_gtag_id, " is detected, but script is not injected because you are using a testing write key."));
        }
        else {
            this.loadGtagScript();
        }
        var event = getStoredIdentify(this.user);
        this.initTag(event);
    };
    GoogleAdsGtag.prototype.loadGtagScript = function () {
        var localWindow = this.browser.window();
        localWindow.dataLayer = localWindow.dataLayer || [];
        localWindow.gtag = function () {
            /* eslint-disable prefer-rest-params */
            localWindow.dataLayer.push(arguments);
        };
        var scriptURL = "https://www.googletagmanager.com/gtag/js?id=".concat(this.settings.google_ads_gtag_id);
        if (!this.browser.scriptAlreadyInPage(scriptURL)) {
            this.browser.injectScript(scriptURL, { async: true });
        }
    };
    GoogleAdsGtag.prototype.initTag = function (identifyEvent) {
        this.setUserData(identifyEvent);
        this.gtag("js", new Date());
        this.gtag("config", this.settings.google_ads_gtag_id);
    };
    GoogleAdsGtag.prototype.setUserData = function (identifyEvent) {
        var userData = this.mapUserData(identifyEvent);
        if (Object.keys(userData).length > 0) {
            this.gtag("set", "user_data", userData);
        }
    };
    GoogleAdsGtag.prototype.mapUserData = function (event) {
        var transformationsMap = {
            sha256_email_address: [trim, toLowerCase],
            sha256_phone_number: [trim, toLowerCase],
            "address.sha256_first_name": [trim, toLowerCase],
            "address.sha256_last_name": [trim, toLowerCase],
        };
        return this.fieldsMapper.mapEvent(event, transformationsMap);
    };
    GoogleAdsGtag.prototype.gtag = function () {
        var _a;
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        if (this.testingMode) {
            this.logger.log("Will call window.gtag with the following params in order:", args);
            return;
        }
        (_a = this.browser.window()).gtag.apply(_a, args);
    };
    // remove the fields that should only be sent on gtag("set") call from properties
    GoogleAdsGtag.prototype.removeUserInitData = function (properties) {
        var output = {};
        for (var key in properties) {
            if (USER_DATA_INIT_FIELDS.has(key)) {
                continue;
            }
            output[key] = properties[key];
        }
        return output;
    };
    return GoogleAdsGtag;
}());
export { GoogleAdsGtag };
// the fields that should only be sent on gtag("set") call
var USER_DATA_INIT_FIELDS = new Set([
    "email",
    "phone_number",
    "address",
    "sha256_email_address",
    "sha256_phone_number",
]);
//# sourceMappingURL=googleAdsGtag.js.map