/* eslint-disable  @typescript-eslint/no-explicit-any */
/* eslint-disable  no-case-declarations */
import { FieldMappingSourceType, TrackingEventType, } from "../plugin";
import _ from "lodash";
import { Liquid } from "liquidjs";
import { applyTransformations } from "./tranformations";
import { JournifyDefaultEvent, JournifyEventType, } from "../../../domain/event";
import * as uuid from "uuid";
var CURRENT_DATE_VAR_NAME = "CURRENT_DATE";
var CURRENT_TIME_VAR_NAME = "CURRENT_TIME";
var UUID_VAR_NAME = "UUID";
var EVENT_TEMPLATING_KEY = "record";
var ARRAY_PATH_SEPARATOR = ".$";
var FieldsMapperFactoryImpl = /** @class */ (function () {
    function FieldsMapperFactoryImpl() {
    }
    FieldsMapperFactoryImpl.prototype.newFieldMapper = function (fieldMappings, now) {
        if (now === void 0) { now = function () { return new Date(); }; }
        return new FieldsMapperImpl(fieldMappings, now);
    };
    return FieldsMapperFactoryImpl;
}());
export { FieldsMapperFactoryImpl };
var FieldsMapperImpl = /** @class */ (function () {
    function FieldsMapperImpl(fieldMappings, now) {
        this.fieldMappings = fieldMappings;
        this.now = now;
        this.liquidEngine = new Liquid();
        this.templateCache = {};
        for (var _i = 0, _a = this.fieldMappings; _i < _a.length; _i++) {
            var mapping = _a[_i];
            if (mapping.source.type == FieldMappingSourceType.TEMPLATE) {
                var tpl = this.liquidEngine.parse(mapping.source.value);
                this.templateCache[mapping.target.name] = tpl;
            }
        }
    }
    FieldsMapperImpl.prototype.mapEvent = function (event, transformationsMap, options) {
        var _a;
        if (options === void 0) { options = {
            ignoreUnmappedProperties: false,
        }; }
        if (!event || Object.keys(event).length === 0) {
            return {};
        }
        var excludedProperties = new Set();
        var properties = {};
        for (var _i = 0, _b = this.fieldMappings; _i < _b.length; _i++) {
            var mapping = _b[_i];
            var value = null;
            switch (mapping.source.type) {
                case FieldMappingSourceType.FIELD:
                    value = this.getValue(event, mapping.source.value);
                    var excludeProp = mapping.source.value.startsWith("properties.");
                    var isArrayPath = this.isArrayPath(mapping.source.value);
                    if (excludeProp && isArrayPath) {
                        excludedProperties.add(mapping.source.value.split(ARRAY_PATH_SEPARATOR)[0]);
                    }
                    else if (excludeProp) {
                        excludedProperties.add(mapping.source.value);
                    }
                    break;
                case FieldMappingSourceType.TEMPLATE:
                    value = this.liquidEngine.renderSync(this.templateCache[mapping.target.name], (_a = {},
                        _a[EVENT_TEMPLATING_KEY] = event,
                        _a));
                    break;
                case FieldMappingSourceType.CONSTANT:
                    value = mapping.source.value;
                    break;
                case FieldMappingSourceType.VARIABLE:
                    value = this.mapVariableValue(mapping);
                    break;
            }
            if (value &&
                transformationsMap &&
                transformationsMap[mapping.target.name]) {
                value = applyTransformations(value, transformationsMap[mapping.target.name]);
            }
            if (value) {
                properties = this.setValue(properties, mapping.target.name, value);
            }
        }
        if (options.ignoreUnmappedProperties) {
            return properties;
        }
        var eventProps = event["properties"];
        if (eventProps) {
            for (var key in eventProps) {
                if (!excludedProperties.has("properties." + key)) {
                    properties[key] = eventProps[key];
                }
            }
        }
        return properties;
    };
    FieldsMapperImpl.prototype.setValue = function (obj, path, value) {
        if (this.isArrayPath(path)) {
            return this.setValues(obj, path, value);
        }
        return _.set(obj, path, value);
    };
    FieldsMapperImpl.prototype.setValues = function (obj, key, sourceValue) {
        var pathParts = key.split(ARRAY_PATH_SEPARATOR);
        var size = pathParts.length;
        // Return if the path is invalid (equals to .$, no .$ found or more than one .$ found)
        if (size === 0 || size > 2 || key === ARRAY_PATH_SEPARATOR) {
            return obj;
        }
        // If there is no nested path, replace the array with the source value
        var arrayPath = pathParts[0];
        if (size === 1 || pathParts[1] === "") {
            return this.setValue(obj, arrayPath, sourceValue);
        }
        // Get the array value from the record
        var value = this.getValue(obj, arrayPath);
        // Create the array if it doesn't exist
        if (value === undefined) {
            value = [];
        }
        // Do nothing if the returned value is not an array (when the key points to a non-array field)
        if (!Array.isArray(value)) {
            return obj;
        }
        var array = value;
        var arraySize = array.length;
        // Set the nested path values (remove the first character which is a dot)
        var nestedPath = pathParts[1].substring(1);
        // Check if sourceValue is an array
        if (Array.isArray(sourceValue)) {
            for (var i = 0; i < sourceValue.length; i++) {
                // Expand the array if needed
                if (i >= arraySize) {
                    array.push({});
                }
                // Set nested values
                if (typeof array[i] === "object" && !Array.isArray(array[i])) {
                    array[i] = this.setValue(array[i], nestedPath, sourceValue[i]);
                }
            }
        }
        else {
            if (arraySize === 0) {
                array.push({});
            }
            for (var i = 0; i < array.length; i++) {
                if (typeof array[i] === "object" && !Array.isArray(array[i])) {
                    array[i] = this.setValue(array[i], nestedPath, sourceValue);
                }
            }
        }
        return this.setValue(obj, arrayPath, array);
    };
    FieldsMapperImpl.prototype.getValue = function (obj, path) {
        if (this.isArrayPath(path)) {
            return this.getValues(obj, path);
        }
        return _.get(obj, path);
    };
    FieldsMapperImpl.prototype.isArrayPath = function (path) {
        return path.includes(ARRAY_PATH_SEPARATOR);
    };
    FieldsMapperImpl.prototype.getValues = function (data, path) {
        var pathParts = path.split(ARRAY_PATH_SEPARATOR);
        var size = pathParts.length;
        // Return false if the path is invalid (no .$ found or more than one .$ found)
        if (size === 0 || size > 2) {
            return null;
        }
        // Get the array value from the data
        var arrayPath = pathParts[0];
        var value = this.getValue(data, arrayPath);
        if (!value) {
            return value;
        }
        // If the returned value is not an array, return false
        if (!Array.isArray(value)) {
            return null;
        }
        // If there is no nested path, return the array as is
        if (size === 1 || pathParts[1] === "") {
            return value;
        }
        // Get the nested path values
        var nestedPath = pathParts[1].substring(1);
        var values = [];
        var keyFound = false;
        for (var _i = 0, value_1 = value; _i < value_1.length; _i++) {
            var v = value_1[_i];
            var nestedValue = this.getValue(v, nestedPath);
            if (nestedValue !== undefined) {
                values.push(nestedValue);
                keyFound = true;
            }
            else {
                // Append null to keep the array size consistent with the original array
                values.push(null);
            }
        }
        if (!keyFound || values.length === 0) {
            return null;
        }
        return values;
    };
    FieldsMapperImpl.prototype.mapVariableValue = function (mapping) {
        switch (mapping.source.value) {
            case CURRENT_DATE_VAR_NAME:
                return this.getCurrentUtcDate();
            case CURRENT_TIME_VAR_NAME:
                return this.getCurrentUtcTime();
            case UUID_VAR_NAME:
                return uuid.v4();
        }
        return "";
    };
    FieldsMapperImpl.prototype.getCurrentUtcDate = function () {
        var currentDate = this.now();
        var year = currentDate.getUTCFullYear();
        var month = String(currentDate.getUTCMonth() + 1).padStart(2, "0"); // Months are 0-indexed
        var day = String(currentDate.getUTCDate()).padStart(2, "0");
        return "".concat(year, "-").concat(month, "-").concat(day);
    };
    FieldsMapperImpl.prototype.getCurrentUtcTime = function () {
        var now = this.now();
        return now.toISOString();
    };
    return FieldsMapperImpl;
}());
var EventMapperFactoryImpl = /** @class */ (function () {
    function EventMapperFactoryImpl() {
    }
    EventMapperFactoryImpl.prototype.newEventMapper = function (eventMappings) {
        return new EventMapperImpl(eventMappings);
    };
    return EventMapperFactoryImpl;
}());
export { EventMapperFactoryImpl };
var EventMapperImpl = /** @class */ (function () {
    function EventMapperImpl(eventMappings) {
        this.pageEventsMapping = {};
        this.trackEventsMapping = {};
        this.groupEventsMapping = {};
        this.identifyEventsMapping = {};
        this.eventMappings = eventMappings;
        this.init();
    }
    EventMapperImpl.prototype.getEventMapping = function (event) {
        switch (event.type) {
            case JournifyEventType.TRACK:
                return this.trackEventsMapping[event.event];
            case JournifyEventType.PAGE:
                return this.pageEventsMapping[event.event || JournifyDefaultEvent.PAGE];
            case JournifyEventType.GROUP:
                return this.groupEventsMapping[event.event || JournifyDefaultEvent.GROUP];
            case JournifyEventType.IDENTIFY:
                return this.identifyEventsMapping[event.event || JournifyDefaultEvent.IDENTIFY];
            default:
                return null;
        }
    };
    EventMapperImpl.prototype.init = function () {
        for (var _i = 0, _a = this.eventMappings; _i < _a.length; _i++) {
            var mapping = _a[_i];
            if (!mapping.enabled) {
                continue;
            }
            var pixelMapping = {
                pixelEventName: mapping.destination_event_key,
                filters: mapping.filters,
            };
            switch (mapping.event_type) {
                case TrackingEventType.TRACK_EVENT:
                    this.trackEventsMapping[mapping.event_name] = pixelMapping;
                    break;
                case TrackingEventType.PAGE_EVENT:
                    this.pageEventsMapping[JournifyDefaultEvent.PAGE] = pixelMapping;
                    break;
                case TrackingEventType.GROUP_EVENT:
                    this.groupEventsMapping[JournifyDefaultEvent.GROUP] = pixelMapping;
                    break;
                case TrackingEventType.IDENTIFY_EVENT:
                    this.identifyEventsMapping[JournifyDefaultEvent.IDENTIFY] =
                        pixelMapping;
                    break;
            }
        }
    };
    return EventMapperImpl;
}());
export { EventMapperImpl };
//# sourceMappingURL=mapping.js.map