var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import { toSettingsObject } from "../lib/settings";
import { matchFilters } from "../lib/filters";
import { toInt } from "../lib/tranformations";
var LINKEDIN_INSIGHT_TAG_SCRIPT_URL = "https://snap.licdn.com/li.lms-analytics/insight.min.js";
var LinkedinAdsInsightTag = /** @class */ (function () {
    function LinkedinAdsInsightTag(deps) {
        this.name = "linkedin_ads_insight_tag";
        this.settings = {};
        this.browser = deps.browser;
        this.fieldsMapper = deps.fieldMapperFactory.newFieldMapper(deps.sync.field_mappings);
        this.testingMode = deps.testingWriteKey;
        this.eventMapper = deps.eventMapperFactory.newEventMapper(deps.sync.event_mappings);
        this.logger = deps.logger;
        this.init(deps.sync);
    }
    LinkedinAdsInsightTag.prototype.group = function (ctx) {
        return this.trackInsightTagEvent(ctx);
    };
    LinkedinAdsInsightTag.prototype.identify = function (ctx) {
        return this.trackInsightTagEvent(ctx);
    };
    LinkedinAdsInsightTag.prototype.page = function (ctx) {
        return this.trackInsightTagEvent(ctx);
    };
    LinkedinAdsInsightTag.prototype.track = function (ctx) {
        return this.trackInsightTagEvent(ctx);
    };
    LinkedinAdsInsightTag.prototype.updateSettings = function (sync) {
        this.init(sync);
    };
    LinkedinAdsInsightTag.prototype.init = function (sync) {
        this.settings = toSettingsObject(sync.settings);
        if (this.testingMode) {
            this.logger.log("Linkedin Insight Tag ".concat(this.settings.linkedin_partner_id, " is detected, but script is not injected because you are using a testing write key."));
        }
        else {
            this.loadTagScript();
        }
    };
    LinkedinAdsInsightTag.prototype.loadTagScript = function () {
        var localWindow = this.browser.window();
        localWindow._linkedin_data_partner_ids =
            window._linkedin_data_partner_ids || [];
        localWindow._linkedin_data_partner_ids.push(this.settings.linkedin_partner_id);
        var linktrk = localWindow.lintrk;
        if (!linktrk) {
            localWindow.lintrk = function (a, b) {
                localWindow.lintrk.q.push([a, b]);
            };
            localWindow.lintrk.q = [];
        }
        this.browser.injectScript(LINKEDIN_INSIGHT_TAG_SCRIPT_URL, { async: true });
    };
    LinkedinAdsInsightTag.prototype.trackInsightTagEvent = function (ctx) {
        var event = ctx.getEvent();
        var eventMapping = this.eventMapper.getEventMapping(event);
        if (!eventMapping || !matchFilters(event, eventMapping === null || eventMapping === void 0 ? void 0 : eventMapping.filters)) {
            return ctx;
        }
        var properties = this.fieldsMapper.mapEvent(event, {}, { ignoreUnmappedProperties: true });
        this.lintrk("track", __assign({ conversion_id: toInt(eventMapping.pixelEventName) }, properties));
        return ctx;
    };
    LinkedinAdsInsightTag.prototype.lintrk = function () {
        var _a;
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        if (this.testingMode) {
            this.logger.log("Will call window.lintrk with the following params in order:", args);
            return;
        }
        (_a = this.browser.window()).lintrk.apply(_a, args);
    };
    return LinkedinAdsInsightTag;
}());
export { LinkedinAdsInsightTag };
//# sourceMappingURL=linkedinAdsInsightTag.js.map