var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
import { EventMapperImpl, } from "../lib/mapping";
import { toSettingsObject } from "../lib/settings";
import { getStoredIdentify } from "../lib/identify";
import { JournifyEventType } from "../../../domain/event";
import { toLowerCase, trim } from "../lib/tranformations";
import { matchFilters } from "../lib/filters";
var PINTEREST_PIXEL_SCRIPT_URL = "https://s.pinimg.com/ct/core.js";
var PinterestTag = /** @class */ (function () {
    function PinterestTag(deps) {
        this.name = "pinterest_tag";
        this.user = deps.user;
        this.eventMapper = deps.eventMapperFactory.newEventMapper(deps.sync.event_mappings);
        this.browser = deps.browser;
        this.fieldMapperFactory = deps.fieldMapperFactory;
        this.testingMode = deps.testingWriteKey;
        this.logger = deps.logger;
        this.init(deps.sync);
    }
    PinterestTag.prototype.identify = function (ctx) {
        var newEvent = ctx.getEvent();
        var storedEvent = getStoredIdentify(this.user);
        var event = {
            type: JournifyEventType.IDENTIFY,
            userId: newEvent.userId || storedEvent.userId,
            anonymousId: newEvent.anonymousId || storedEvent.anonymousId,
            traits: __assign(__assign({}, (storedEvent.traits || {})), (newEvent.traits || {})),
        };
        this.initTag(event);
        return this.trackTagEvent(ctx);
    };
    PinterestTag.prototype.page = function (ctx) {
        return this.trackTagEvent(ctx);
    };
    PinterestTag.prototype.track = function (ctx) {
        return this.trackTagEvent(ctx);
    };
    PinterestTag.prototype.group = function (ctx) {
        return this.trackTagEvent(ctx);
    };
    PinterestTag.prototype.updateSettings = function (sync) {
        this.init(sync);
    };
    PinterestTag.prototype.init = function (sync) {
        this.fieldsMapper = this.fieldMapperFactory.newFieldMapper(sync.field_mappings);
        this.eventMapper = new EventMapperImpl(sync.event_mappings);
        this.settings = toSettingsObject(sync.settings);
        if (this.testingMode) {
            this.logger.log("Pinterest Tag ".concat(this.settings.tag_id, " is detected, but script is not injected because you are using a testing write key."));
        }
        else {
            this.loadScript();
        }
        var event = getStoredIdentify(this.user);
        this.initTag(event);
    };
    PinterestTag.prototype.loadScript = function () {
        var localWindow = this.browser.window();
        if (localWindow.pintrk) {
            return;
        }
        // eslint-disable-next-line @typescript-eslint/ban-ts-comment
        // @ts-ignore
        localWindow.pintrk = function () {
            // eslint-disable-next-line prefer-rest-params
            localWindow.pintrk.queue.push(Array.prototype.slice.call(arguments));
        };
        var localDocument = this.browser.document();
        var n = localWindow.pintrk;
        (n.queue = []), (n.version = "3.0");
        var t = localDocument.createElement("script");
        (t.async = !0), (t.src = PINTEREST_PIXEL_SCRIPT_URL);
        var r = localDocument.getElementsByTagName("script")[0];
        r.parentNode.insertBefore(t, r);
    };
    PinterestTag.prototype.initTag = function (identifyEvent) {
        var transformationsMap = {
            em: [trim, toLowerCase],
        };
        var userData = this.fieldsMapper.mapEvent(identifyEvent, transformationsMap, { ignoreUnmappedProperties: true });
        this.callPintrk("load", this.settings.tag_id, userData);
    };
    PinterestTag.prototype.callPintrk = function (trackingType) {
        var _a;
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        if (this.testingMode) {
            this.logger.log("Will call window.pintrk with the following params in order:", __spreadArray([trackingType], args, true));
        }
        else {
            (_a = this.browser.window()).pintrk.apply(_a, __spreadArray([trackingType], args, false));
        }
    };
    PinterestTag.prototype.trackTagEvent = function (ctx) {
        var event = ctx.getEvent();
        var eventMapping = this.eventMapper.getEventMapping(event);
        if (!eventMapping || !matchFilters(event, eventMapping === null || eventMapping === void 0 ? void 0 : eventMapping.filters)) {
            return ctx;
        }
        var mappedProperties = this.fieldsMapper.mapEvent(event, {}, { ignoreUnmappedProperties: true });
        var eventName = (eventMapping === null || eventMapping === void 0 ? void 0 : eventMapping.pixelEventName) || event.event;
        if (!eventMapping) {
            return ctx;
        }
        this.callPintrk("track", eventName, mappedProperties);
        return ctx;
    };
    return PinterestTag;
}());
export { PinterestTag };
//# sourceMappingURL=pinterestTag.js.map