var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
import { matchFilters } from "../lib/filters";
import { JournifyEventType } from "../../../domain/event";
import { toLowerCase, trim } from "../lib/tranformations";
import { getStoredIdentify } from "../lib/identify";
import { toSettingsObject } from "../lib/settings";
var SNAPCHAT_PIXEL_SCRIPT_URL = "https://sc-static.net/scevent.min.js";
var SnapchatPixel = /** @class */ (function () {
    function SnapchatPixel(deps) {
        this.name = "snapchat_pixel";
        this.user = deps.user;
        this.browser = deps.browser;
        this.eventMapperFactory = deps.eventMapperFactory;
        this.fieldMapperFactory = deps.fieldMapperFactory;
        this.testingMode = deps.testingWriteKey;
        this.logger = deps.logger;
        this.init(deps.sync);
    }
    SnapchatPixel.prototype.identify = function (ctx) {
        var newEvent = ctx.getEvent();
        var storedEvent = getStoredIdentify(this.user);
        var event = {
            type: JournifyEventType.IDENTIFY,
            userId: newEvent.userId || storedEvent.userId,
            anonymousId: newEvent.anonymousId || storedEvent.anonymousId,
            traits: __assign(__assign({}, (storedEvent.traits || {})), (newEvent.traits || {})),
        };
        this.initPixel(event);
        return this.trackPixelEvent(ctx);
    };
    SnapchatPixel.prototype.track = function (ctx) {
        return this.trackPixelEvent(ctx);
    };
    SnapchatPixel.prototype.group = function (ctx) {
        return this.trackPixelEvent(ctx);
    };
    SnapchatPixel.prototype.updateSettings = function (sync) {
        this.init(sync);
    };
    SnapchatPixel.prototype.page = function (ctx) {
        return this.trackPixelEvent(ctx);
    };
    SnapchatPixel.prototype.trackPixelEvent = function (ctx) {
        var event = ctx.getEvent();
        var eventMapping = this.eventMapper.getEventMapping(event);
        if (!eventMapping || !matchFilters(event, eventMapping === null || eventMapping === void 0 ? void 0 : eventMapping.filters)) {
            return ctx;
        }
        var mappedProperties = this.fieldsMapper.mapEvent(event);
        var eventName = (eventMapping === null || eventMapping === void 0 ? void 0 : eventMapping.pixelEventName) || event.event;
        var properties = this.transformProperties(mappedProperties);
        this.callPixelHelper("track", eventName, properties);
        return ctx;
    };
    SnapchatPixel.prototype.transformProperties = function (properties) {
        if (properties.item_category) {
            if (properties.item_category instanceof Array) {
                if (properties.item_category.length > 1) {
                    properties.item_category = [properties.item_category.join(",")];
                }
                if (properties.item_category.length === 1) {
                    properties.item_category = properties.item_category[0];
                }
            }
        }
        if (properties.item_ids instanceof Array &&
            properties.item_ids.length > 0) {
            properties.item_ids = [properties.item_ids.join(";")];
        }
        if (properties.number_items instanceof Array &&
            properties.number_items.length > 0) {
            properties.number_items = properties.number_items.join(";");
        }
        if (properties.transaction_id) {
            properties.transaction_id = properties.transaction_id.toString();
        }
        return properties;
    };
    SnapchatPixel.prototype.init = function (sync) {
        this.eventMapper = this.eventMapperFactory.newEventMapper(sync.event_mappings);
        this.fieldsMapper = this.fieldMapperFactory.newFieldMapper(sync.field_mappings);
        this.settings = toSettingsObject(sync.settings);
        if (this.testingMode) {
            this.logger.log("Snapchat Pixel ".concat(this.settings.pixel_id, " is detected, but script is not injected because you are using a testing write key."));
        }
        else {
            this.loadScript();
        }
        var event = getStoredIdentify(this.user);
        this.initPixel(event);
    };
    SnapchatPixel.prototype.initPixel = function (identifyEvent) {
        var userData = this.mapUserData(identifyEvent);
        this.callPixelHelper("init", this.settings.pixel_id, userData);
    };
    SnapchatPixel.prototype.callPixelHelper = function (trackingType) {
        var _a;
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        if (this.testingMode) {
            this.logger.log("Will call window.snaptr with the following params in order:", __spreadArray([trackingType], args, true));
        }
        else {
            (_a = this.browser.window()).snaptr.apply(_a, __spreadArray([trackingType], args, false));
        }
    };
    SnapchatPixel.prototype.mapUserData = function (event) {
        var transformationsMap = {
            user_email: [trim, toLowerCase],
            ip_address: [trim, toLowerCase],
            user_hashed_email: [trim, toLowerCase],
            user_hashed_phone_number: [trim, toLowerCase],
            firstname: [trim, toLowerCase],
            lastname: [trim, toLowerCase],
            geo_city: [trim, toLowerCase],
            geo_region: [trim, toLowerCase],
            geo_postal_code: [trim, toLowerCase],
            geo_country: [trim, toLowerCase],
            age: [trim],
        };
        return this.fieldsMapper.mapEvent(event, transformationsMap);
    };
    /* eslint-disable */
    SnapchatPixel.prototype.loadScript = function () {
        var localWindow = this.browser.window();
        if (localWindow.snaptr)
            return;
        var a = (localWindow.snaptr = function () {
            a.handleRequest
                ? a.handleRequest.apply(a, arguments)
                : a.queue.push(arguments);
        });
        a.queue = [];
        var s = "script";
        var localDocument = this.browser.document();
        var r = localDocument.createElement(s);
        r.async = !0;
        r.src = SNAPCHAT_PIXEL_SCRIPT_URL;
        var u = localDocument.getElementsByTagName(s)[0];
        u.parentNode.insertBefore(r, u);
    };
    return SnapchatPixel;
}());
export { SnapchatPixel };
//# sourceMappingURL=snapchatPixel.js.map