var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
/* eslint-disable  @typescript-eslint/no-explicit-any */
import { JournifyEventType } from "../../../domain/event";
import { matchFilters } from "../lib/filters";
import { EventMapperImpl, } from "../lib/mapping";
import { toLowerCase, trim } from "../lib/tranformations";
import { toSettingsObject } from "../lib/settings";
import { getStoredIdentify } from "../lib/identify";
var TIKTOK_SCRIPT_URL = "https://analytics.tiktok.com/i18n/pixel/events.js";
var TikTokPixel = /** @class */ (function () {
    function TikTokPixel(deps) {
        this.name = "tiktok_pixel";
        this.user = deps.user;
        this.eventMapper = deps.eventMapperFactory.newEventMapper(deps.sync.event_mappings);
        this.browser = deps.browser;
        this.fieldMapperFactory = deps.fieldMapperFactory;
        this.testingMode = deps.testingWriteKey;
        this.logger = deps.logger;
        this.init(deps.sync);
    }
    TikTokPixel.prototype.identify = function (ctx) {
        var newEvent = ctx.getEvent();
        var storedEvent = getStoredIdentify(this.user);
        var event = {
            type: JournifyEventType.IDENTIFY,
            userId: newEvent.userId || storedEvent.userId,
            anonymousId: newEvent.anonymousId || storedEvent.anonymousId,
            traits: __assign(__assign({}, (storedEvent.traits || {})), (newEvent.traits || {})),
        };
        var userData = this.mapUserData(event);
        this.callPixelHelper(event.type, userData);
        return this.trackPixelEvent(ctx);
    };
    TikTokPixel.prototype.track = function (ctx) {
        return this.trackPixelEvent(ctx);
    };
    TikTokPixel.prototype.page = function (ctx) {
        return this.trackPixelEvent(ctx);
    };
    TikTokPixel.prototype.group = function (ctx) {
        return this.trackPixelEvent(ctx);
    };
    TikTokPixel.prototype.updateSettings = function (sync) {
        this.init(sync);
    };
    TikTokPixel.prototype.init = function (sync) {
        this.fieldsMapper = this.fieldMapperFactory.newFieldMapper(sync.field_mappings, function () { return new Date(); });
        this.eventMapper = new EventMapperImpl(sync.event_mappings);
        this.settings = toSettingsObject(sync.settings);
        if (this.testingMode) {
            this.logger.log("Tiktok Pixel ".concat(this.settings.pixel_code, " is detected, but script is not injected because you are using a testing write key."));
        }
        else {
            this.loadTiktokPixel();
        }
    };
    TikTokPixel.prototype.mapUserData = function (event) {
        var transformationsMap = {
            email: [trim, toLowerCase],
            phone_number: [trim],
            external_id: [trim],
        };
        return this.fieldsMapper.mapEvent(event, transformationsMap);
    };
    TikTokPixel.prototype.callPixelHelper = function (eventType, args, eventId) {
        if (eventId === void 0) { eventId = ""; }
        var event = __assign(__assign({}, args), { pixelCode: this.settings.pixel_code });
        switch (eventType) {
            case JournifyEventType.IDENTIFY:
                if (this.testingMode) {
                    return this.logger.log("Will call window.ttq.identify with the following params in order:", __assign(__assign({}, event), { event_id: eventId }));
                }
                window.ttq.identify(__assign(__assign({}, event), { event_id: eventId }));
                break;
            case JournifyEventType.PAGE:
                if (args.event === "Pageview") {
                    if (this.testingMode) {
                        return this.logger.log("Will call window.ttq.page with the following params in order:", __assign(__assign({}, event), { event_id: eventId }));
                    }
                    window.ttq.page(__assign(__assign({}, event), { event_id: eventId }));
                }
                else {
                    if (this.testingMode) {
                        return this.logger.log("Will call window.ttq.track with the following params in order:", args.event, event, { event_id: eventId });
                    }
                    window.ttq.track(args.event, event, { event_id: eventId });
                }
                break;
            default:
                if (this.testingMode) {
                    return this.logger.log("Will call window.ttq.track with the following params in order:", args.event, event, { event_id: eventId });
                }
                window.ttq.track(args.event, event, { event_id: eventId });
                break;
        }
    };
    TikTokPixel.prototype.trackPixelEvent = function (ctx) {
        var event = ctx.getEvent();
        var eventMapping = this.eventMapper.getEventMapping(event);
        if (!eventMapping || !matchFilters(event, eventMapping === null || eventMapping === void 0 ? void 0 : eventMapping.filters)) {
            return ctx;
        }
        var mappedProperties = this.fieldsMapper.mapEvent(event);
        var eventId = mappedProperties.event_id;
        delete mappedProperties.event_id;
        var eventName = (eventMapping === null || eventMapping === void 0 ? void 0 : eventMapping.pixelEventName) || event.event;
        this.callPixelHelper(event.type, __assign(__assign({}, mappedProperties), { event: eventName }), eventId);
        return ctx;
    };
    /* eslint-disable */
    TikTokPixel.prototype.loadTiktokPixel = function () {
        var w = this.browser.window();
        var d = this.browser.document();
        var t = "ttq";
        w.TiktokAnalyticsObject = t;
        var ttq = (w[t] = w[t] || []);
        (ttq.methods = [
            "page",
            "track",
            "identify",
            "instances",
            "debug",
            "on",
            "off",
            "once",
            "ready",
            "alias",
            "group",
            "enableCookie",
            "disableCookie",
        ]),
            (ttq.setAndDefer = function (t, e) {
                t[e] = function () {
                    t.push([e].concat(Array.prototype.slice.call(arguments, 0)));
                };
            });
        for (var i = 0; i < ttq.methods.length; i++)
            ttq.setAndDefer(ttq, ttq.methods[i]);
        (ttq.instance = function (t) {
            for (var e = ttq._i[t] || [], n = 0; n < ttq.methods.length; n++)
                ttq.setAndDefer(e, ttq.methods[n]);
            return e;
        }),
            (ttq.load = function (e, n) {
                var i = TIKTOK_SCRIPT_URL;
                (ttq._i = ttq._i || {}),
                    (ttq._i[e] = []),
                    (ttq._i[e]._u = i),
                    (ttq._t = ttq._t || {}),
                    (ttq._t[e] = +new Date()),
                    (ttq._o = ttq._o || {}),
                    (ttq._partner = ttq._partner || "Journify"),
                    (ttq._o[e] = n || {});
                var o = d.createElement("script");
                (o.type = "text/javascript"),
                    (o.async = !0),
                    (o.src = i + "?sdkid=" + e + "&lib=" + t);
                var a = d.getElementsByTagName("script")[0];
                a.parentNode.insertBefore(o, a);
            });
        ttq.load(this.settings.pixel_code);
    };
    return TikTokPixel;
}());
export { TikTokPixel };
//# sourceMappingURL=tiktokPixel.js.map