var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { Sdk } from "./sdk";
import { NullStore } from "../store/nullStore";
import { CookiesStore } from "../store/cookiesStore";
import { MemoryStore } from "../store/memoryStore";
import { EventQueueImpl } from "../transport/queue";
import { UserImpl } from "../domain/user";
import { EventFactoryImpl } from "../transport/eventFactory";
import { ContextFactoryImpl } from "../transport/context";
import { OperationsPriorityQueueImpl } from "../lib/priorityQueue";
import { GroupFactoryImpl } from "../domain/group";
import { SESSION_ID_PERSISTENCE_KEY, StoresGroup } from "../store/store";
import { BrowserStore } from "../store/browserStore";
import { FacebookPixel } from "../transport/plugins/facebook/facebookPixel";
import { SnapchatPixel } from "../transport/plugins/snapchat/snapchatPixel";
import { BrowserImpl, UTM_KEYS } from "../transport/browser";
import { GA4Pixel } from "../transport/plugins/ga4_pixel/ga4PPixel";
import { TikTokPixel } from "../transport/plugins/tiktok/tiktokPixel";
import { CleverTapPlugin } from "../transport/plugins/cleverTap/cleverTapPlugin";
import { CleverTapWrapperImpl } from "../transport/plugins/cleverTap/cleverTapWrapper";
import { EventMapperFactoryImpl, FieldsMapperFactoryImpl, } from "../transport/plugins/lib/mapping";
import { SessionStore } from "../store/sessionStore";
import { ExternalIdsSessionCacheImpl } from "../domain/externalId";
import { AutoCapturePII } from "../lib/autoCapturePII";
import { HttpCookieServiceImpl } from "../lib/httpCookieService";
import { PinterestTag } from "../transport/plugins/pinterest/pinterestTag";
import { JournifyioPlugin } from "../transport/plugins/journifyio/journifyio";
import { XPixel } from "../transport/plugins/x/xPixel";
import { BingAdsTag } from "../transport/plugins/bing_ads_tag/bing_ads_tag";
import { GoogleAdsGtag } from "../transport/plugins/google_ads_gtag/googleAdsGtag";
import { LinkedinAdsInsightTag } from "../transport/plugins/linkedin_ads_insight_tag/linkedinAdsInsightTag";
const INTEGRATION_PLUGINS = {
    bing_ads_tag: BingAdsTag,
    clevertap: CleverTapPlugin,
    facebook_pixel: FacebookPixel,
    ga4_pixel: GA4Pixel,
    google_ads_gtag: GoogleAdsGtag,
    linkedin_ads_insight_tag: LinkedinAdsInsightTag,
    pinterest_tag: PinterestTag,
    snapchat_pixel: SnapchatPixel,
    tiktok_pixel: TikTokPixel,
    x_pixel: XPixel,
};
const JOURNIFY_PREFIX = "[Journify]";
export class Loader {
    constructor(sentryWrapper) {
        this.sdk = null;
        this.user = null;
        this.plugins = {};
        this.sessionIntervalId = null;
        this.stores = null;
        this.cookiesStore = null;
        this.sentryWrapper = sentryWrapper;
    }
    load(sdkConfig, writeKeySettings) {
        var _a, _b, _c, _d, _e, _f;
        return __awaiter(this, void 0, void 0, function* () {
            this.sdkSettings = sdkConfig;
            this.writeKeySettings = writeKeySettings;
            this.startNewSession();
            const browser = new BrowserImpl();
            let cookieService;
            if ((_b = (_a = this.sdkSettings) === null || _a === void 0 ? void 0 : _a.options) === null || _b === void 0 ? void 0 : _b.httpCookieServiceOptions) {
                cookieService = new HttpCookieServiceImpl((_d = (_c = this.sdkSettings) === null || _c === void 0 ? void 0 : _c.options) === null || _d === void 0 ? void 0 : _d.httpCookieServiceOptions, browser, this.sentryWrapper, this.cookiesStore);
            }
            this.user = new UserImpl(this.stores, this.sentryWrapper, cookieService, (_f = (_e = this.sdkSettings) === null || _e === void 0 ? void 0 : _e.options) === null || _f === void 0 ? void 0 : _f.phoneCountryCode);
            yield this.user.load();
            if (!this.sdk) {
                this.initSdk();
            }
            else {
                this.updatePluginSettings();
            }
            return this.sdk;
        });
    }
    initSdk() {
        var _a, _b, _c;
        const fieldMapperFactory = new FieldsMapperFactoryImpl();
        const browser = new BrowserImpl();
        const logger = {
            log: (...args) => console.log(JOURNIFY_PREFIX, ...args),
        };
        const testingMode = isTestingWriteKey(this.sdkSettings.writeKey);
        const pluginExternalSDKs = {
            clevertap: new CleverTapWrapperImpl(browser, logger, testingMode),
        };
        this.plugins = {
            journifyio: new JournifyioPlugin(this.sdkSettings, this.sentryWrapper),
        };
        for (const sync of this.writeKeySettings.syncs) {
            const plugin = INTEGRATION_PLUGINS[sync.destination_app];
            if (plugin) {
                const pluginDeps = {
                    user: this.user,
                    fieldMapperFactory: fieldMapperFactory,
                    eventMapperFactory: new EventMapperFactoryImpl(),
                    browser: browser,
                    sync: sync,
                    testingWriteKey: testingMode,
                    externalSDK: pluginExternalSDKs[sync.destination_app],
                    enableHashing: (_b = (_a = this.sdkSettings) === null || _a === void 0 ? void 0 : _a.options) === null || _b === void 0 ? void 0 : _b.enableHashing,
                    logger: logger,
                    sentry: this.sentryWrapper,
                };
                this.plugins[sync.id] = new plugin(pluginDeps);
            }
        }
        const pQueue = new OperationsPriorityQueueImpl(DEFAULT_MAX_QUEUE_ATTEMPTS);
        const sessionStore = new SessionStore(browser);
        const externalIdsSessionCache = new ExternalIdsSessionCacheImpl(browser, sessionStore);
        const deps = {
            user: this.user,
            eventFactory: new EventFactoryImpl(this.stores, this.cookiesStore, browser, externalIdsSessionCache),
            groupFactory: new GroupFactoryImpl(this.stores),
            contextFactory: new ContextFactoryImpl(),
            eventQueue: new EventQueueImpl(Object.values(this.plugins), pQueue, browser, this.sentryWrapper),
        };
        this.sdk = new Sdk(this.sdkSettings, deps);
        if ((_c = this.sdkSettings.options) === null || _c === void 0 ? void 0 : _c.autoCapturePII) {
            const autoCapturePII = new AutoCapturePII(browser, this.user, this.sdkSettings.options.autoCapturePhoneRegex);
            autoCapturePII.listen();
        }
    }
    updatePluginSettings() {
        const syncsMap = {};
        for (const sync of this.writeKeySettings.syncs) {
            syncsMap[sync.id] = sync;
        }
        for (const syncID in this.plugins) {
            let settings = syncsMap[syncID];
            if (!settings) {
                settings = this.sdkSettings;
            }
            const plugin = this.plugins[syncID];
            plugin.updateSettings(settings);
        }
    }
    initStores() {
        var _a, _b, _c;
        if (this.stores) {
            return;
        }
        const localStore = new BrowserStore(BrowserStore.isLocalStorageAvailable() ? localStorage : undefined);
        const memoryStore = new MemoryStore();
        const cookiesStore = CookiesStore.isAvailable()
            ? new CookiesStore((_c = (_b = (_a = this.sdkSettings) === null || _a === void 0 ? void 0 : _a.options) === null || _b === void 0 ? void 0 : _b.cookie) === null || _c === void 0 ? void 0 : _c.domain)
            : new NullStore();
        this.stores = new StoresGroup(localStore, cookiesStore, memoryStore);
        this.cookiesStore = cookiesStore;
    }
    startNewSession() {
        var _a, _b;
        this.initStores();
        if (this.sessionIntervalId) {
            clearInterval(this.sessionIntervalId);
        }
        const currentEpoch = new Date().getTime();
        if (!this.stores.get(SESSION_ID_PERSISTENCE_KEY)) {
            this.stores.set(SESSION_ID_PERSISTENCE_KEY, currentEpoch);
            const sessionDurationMin = ((_b = (_a = this.sdkSettings) === null || _a === void 0 ? void 0 : _a.options) === null || _b === void 0 ? void 0 : _b.sessionDurationMin) ||
                DEFAULT_SESSION_DURATION_MIN;
            this.sessionIntervalId = setInterval(() => {
                const newSessionId = new Date().getTime();
                this.stores.set(SESSION_ID_PERSISTENCE_KEY, newSessionId);
                this.resetUtmCampaign();
            }, sessionDurationMin * 60 * 1000);
        }
    }
    resetUtmCampaign() {
        UTM_KEYS.forEach((key) => this.stores.remove(key[0]));
    }
}
const WRITE_KEY_TEST_PREFIX = "wk_test_";
const WRITE_KEY_PROD_PREFIX = "wk_";
function isTestingWriteKey(writeKey) {
    return writeKey === null || writeKey === void 0 ? void 0 : writeKey.startsWith(WRITE_KEY_TEST_PREFIX);
}
export function getProductionWriteKey(writeKey) {
    if (isTestingWriteKey(writeKey)) {
        return writeKey === null || writeKey === void 0 ? void 0 : writeKey.replace(WRITE_KEY_TEST_PREFIX, WRITE_KEY_PROD_PREFIX);
    }
    return writeKey;
}
const DEFAULT_MAX_QUEUE_ATTEMPTS = 2;
const DEFAULT_SESSION_DURATION_MIN = 30;
//# sourceMappingURL=loader.js.map