var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { EmitterImpl } from "../transport/emitter";
const IDENTIFY_EVENT_NAME = "identify";
const TRACK_EVENT_NAME = "track";
const PAGE_EVENT_NAME = "page";
const GROUP_EVENT_NAME = "group";
export class Sdk extends EmitterImpl {
    constructor(sdkSettings, deps) {
        super();
        this.sdkSettings = sdkSettings;
        this.contextFactory = deps.contextFactory;
        this.eventQueue = deps.eventQueue;
        this.eventFactory = deps.eventFactory;
        this.user = deps.user;
        this.eventFactory.setUser(this.user);
        this._group = deps.groupFactory.loadGroup();
        this.eventFactory.setGroup(this._group);
    }
    identify(userId, traits, externalIds) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.user.identify(userId, traits, externalIds);
            const event = yield this.eventFactory.newIdentifyEvent();
            const ctx = yield this.dispatchEvent(event);
            const ctxEvent = ctx.getEvent();
            this.emit(IDENTIFY_EVENT_NAME, ctxEvent.userId, ctxEvent.traits);
            return ctx;
        });
    }
    track(eventName, properties, traits) {
        return __awaiter(this, void 0, void 0, function* () {
            if (isNonValidString(eventName)) {
                throw new Error("Event name is missing");
            }
            const event = yield this.eventFactory.newTrackEvent(eventName, properties, traits);
            const ctx = yield this.dispatchEvent(event);
            const ctxEvent = ctx.getEvent();
            this.emit(TRACK_EVENT_NAME, ctxEvent.event, ctxEvent.properties, ctxEvent.traits);
            return ctx;
        });
    }
    page(pageNameParam, properties, traits) {
        return __awaiter(this, void 0, void 0, function* () {
            let pageName = pageNameParam;
            if (isNonValidString(pageName)) {
                pageName = document.title;
            }
            const event = yield this.eventFactory.newPageEvent(pageName, properties, traits);
            const ctx = yield this.dispatchEvent(event);
            const ctxEvent = ctx.getEvent();
            this.emit(PAGE_EVENT_NAME, ctxEvent.event, ctxEvent.properties);
            return ctx;
        });
    }
    group(groupId, traits) {
        return __awaiter(this, void 0, void 0, function* () {
            this._group.identify(groupId, traits);
            const event = yield this.eventFactory.newGroupEvent();
            const ctx = yield this.dispatchEvent(event);
            const ctxEvent = ctx.getEvent();
            this.emit(GROUP_EVENT_NAME, ctxEvent.context.groupId, ctxEvent.traits);
            return ctx;
        });
    }
    dispatchEvent(event) {
        return __awaiter(this, void 0, void 0, function* () {
            const eventCtx = this.contextFactory.newContext(event);
            return this.eventQueue.deliver(eventCtx);
        });
    }
}
function isNonValidString(str) {
    return !str || typeof str !== "string" || str.trim().length === 0;
}
//# sourceMappingURL=sdk.js.map