var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
const supportedCookies = ["_ga"];
export class HttpCookieServiceImpl {
    constructor(options, browser, sentry, cookiesStore) {
        var _a, _b, _c, _d;
        this.initialCookies = {};
        if (!options.renewUrl) {
            throw new Error("Missing required renewUrl option for HttpCookieService");
        }
        this.cookiesStore = cookiesStore;
        this.sentry = sentry;
        const origin = browser.location().origin;
        this.renewUrl = new URL(options.renewUrl, origin).href;
        this.backoff = (_a = options.backoff) !== null && _a !== void 0 ? _a : 300;
        this.retries = (_b = options.retries) !== null && _b !== void 0 ? _b : 2;
        this.timeout = (_c = options.timeout) !== null && _c !== void 0 ? _c : 3000;
        this.pollingInterval = (_d = options.pollingInterval) !== null && _d !== void 0 ? _d : 300;
        if (options.enablePolling) {
            // Gather initial cookies
            this.setInitialCookies();
            // Start polling for cookies changes (when GA4 is loaded and _ga cookie is set)
            this.pollingCookieKeeper(this.pollingInterval);
        }
    }
    pollingCookieKeeper(timeout) {
        setTimeout(() => __awaiter(this, void 0, void 0, function* () {
            this.pollingCookieKeeper(timeout * 2); // Exponential backoff
            // If cookies have not changed, we don't need to do anything.
            if (!this.hasCookiesChanged()) {
                return;
            }
            // When cookies have changed, and we have initial cookies, we need to revert to initial cookies.
            if (!this.isInitialCookiesEmpty()) {
                this.revertInitialCookies();
                return;
            }
            // When cookies have changed, and we have no initial cookies, we need to persist the new cookies to the server.
            yield this.dispatchRenew();
            this.setInitialCookies();
        }), timeout);
    }
    revertInitialCookies() {
        supportedCookies.forEach((cookie) => {
            this.cookiesStore.set(cookie, this.initialCookies[cookie]);
        });
    }
    isInitialCookiesEmpty() {
        return supportedCookies.every((cookie) => this.initialCookies[cookie] === null ||
            this.initialCookies[cookie] === undefined);
    }
    hasCookiesChanged() {
        return supportedCookies.some((cookie) => {
            return this.initialCookies[cookie] !== this.cookiesStore.get(cookie);
        });
    }
    setInitialCookies() {
        supportedCookies.forEach((cookie) => {
            this.initialCookies[cookie] = this.cookiesStore.get(cookie);
        });
    }
    sendHTTPCookies(serviceUrl) {
        return () => __awaiter(this, void 0, void 0, function* () {
            const controller = new AbortController();
            setTimeout(() => controller.abort(), this.timeout);
            const response = yield fetch(serviceUrl, {
                signal: controller.signal,
                credentials: "include",
                headers: {
                    Accept: "application/json",
                    "Content-Type": "application/json",
                },
                method: "POST",
            });
            if (!response.ok) {
                const err = new Error(`Failed to request HTTP Cookies, reason: ${response.statusText}`);
                yield this.sentry.setResponse({
                    url: serviceUrl,
                    headers: response.headers,
                    status: response.status,
                    body: yield response.text(),
                });
                throw err;
            }
            const data = yield response.json();
            const httpCookie = {
                anonymousId: data === null || data === void 0 ? void 0 : data.journifyio_anonymous_id,
                userId: data === null || data === void 0 ? void 0 : data.journifyio_user_id,
            };
            return httpCookie;
        });
    }
    dispatchRenew() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield retry(this.sendHTTPCookies(this.renewUrl), this.retries, this.backoff);
        });
    }
}
function sleep(delayMS) {
    return __awaiter(this, void 0, void 0, function* () {
        return new Promise((resolve) => setTimeout(resolve, delayMS));
    });
}
function retry(req, retries, backoff) {
    return __awaiter(this, void 0, void 0, function* () {
        while (retries >= 0) {
            try {
                return yield req();
            }
            catch (error) {
                if (retries <= 0)
                    throw error;
                retries -= 1;
                yield sleep(backoff);
            }
        }
    });
}
//# sourceMappingURL=httpCookieService.js.map