import { EmitterImpl } from "../transport/emitter";
export class OperationsPriorityQueueImpl extends EmitterImpl {
    constructor(maxAttempts) {
        super();
        this.maxAttempts = maxAttempts;
        this.nowOperations = [];
        this.delayedOperations = [];
        this.attempts = {};
    }
    push(...operations) {
        const accepted = operations.map((operation) => {
            const attempts = this.newAttempt(operation);
            if (attempts > this.maxAttempts || this.alreadyInQueue(operation)) {
                return false;
            }
            this.nowOperations.push(operation);
            return true;
        });
        this.nowOperations = this.nowOperations.sort((a, b) => this.getAttempts(a) - this.getAttempts(b));
        return accepted;
    }
    pushWithBackoff(operation) {
        if (this.getAttempts(operation) === 0) {
            return this.push(operation)[0];
        }
        const attempts = this.newAttempt(operation);
        if (attempts > this.maxAttempts || this.alreadyInQueue(operation)) {
            return false;
        }
        this.delayedOperations.push(operation);
        const delayMs = this.backoffDelayInMs(attempts - 1);
        setTimeout(() => {
            this.nowOperations.push(operation);
            this.delayedOperations = this.delayedOperations.filter((f) => f.getId() !== operation.getId());
            this.emit(ON_OPERATION_DELAY_FINISH);
        }, delayMs);
        return true;
    }
    isEmpty() {
        return this.nowOperations.length === 0;
    }
    pop() {
        return this.nowOperations.shift();
    }
    backoffDelayInMs(attempt) {
        const random = Math.random() + 1;
        const minTimeout = 500;
        const factor = 2;
        return random * minTimeout * Math.pow(factor, attempt);
    }
    newAttempt(operation) {
        this.attempts[operation.getId()] = this.getAttempts(operation) + 1;
        return this.getAttempts(operation);
    }
    getAttempts(operation) {
        var _a;
        return (_a = this.attempts[operation.getId()]) !== null && _a !== void 0 ? _a : 0;
    }
    alreadyInQueue(operation) {
        return (this.nowOperations.includes(operation) ||
            this.delayedOperations.includes(operation) ||
            Boolean(this.nowOperations.find((i) => i.getId() === operation.getId())) ||
            Boolean(this.delayedOperations.find((i) => i.getId() === operation.getId())));
    }
}
export const ON_OPERATION_DELAY_FINISH = "onOperationDelayFinish";
//# sourceMappingURL=priorityQueue.js.map