import { MemoryStore } from "./memoryStore";
export class BrowserStore {
    constructor(storage) {
        this.browserStorage = null;
        this.memoryStore = null;
        if (storage) {
            this.browserStorage = storage;
        }
        else {
            this.memoryStore = new MemoryStore();
        }
    }
    static isLocalStorageAvailable() {
        const testKey = "journify.io-test-browser-storage-key";
        try {
            localStorage.setItem(testKey, "journify.io-test-browser-storage-value");
            localStorage.removeItem(testKey);
            return true;
        }
        catch (e) {
            return false;
        }
    }
    get(key) {
        const val = this.browserStorage
            ? this.browserStorage.getItem(key)
            : this.memoryStore.get(key);
        if (val) {
            try {
                return JSON.parse(val);
            }
            catch (e) {
                return JSON.parse(JSON.stringify(val));
            }
        }
        return null;
    }
    set(key, value) {
        try {
            let persistedValue;
            switch (typeof value) {
                case "string":
                case "number":
                case "boolean":
                    persistedValue = value.toString();
                    break;
                default:
                    persistedValue = JSON.stringify(value);
                    break;
            }
            this.browserStorage
                ? this.browserStorage.setItem(key, persistedValue)
                : this.memoryStore.set(key, persistedValue);
        }
        catch (e) {
            console.warn(`Unable to set ${key} in browser storage, storage may be full.`);
            console.warn(e);
            return null;
        }
        return value;
    }
    remove(key) {
        try {
            this.browserStorage
                ? this.browserStorage.removeItem(key)
                : this.memoryStore.remove(key);
        }
        catch (e) {
            console.warn(`Unable to remove ${key} from browser storage.`);
            console.warn(e);
        }
    }
}
//# sourceMappingURL=browserStore.js.map