import JsCookie from "js-cookie";
const ONE_YEAR = 365;
export class CookiesStore {
    constructor(domain) {
        this.domain = domain;
    }
    static isAvailable() {
        let cookieEnabled = navigator === null || navigator === void 0 ? void 0 : navigator.cookieEnabled;
        if (!cookieEnabled) {
            const testKey = "journify.io-test-cookie-key";
            JsCookie.set(testKey, "journify.io-test-cookie-value");
            cookieEnabled = document.cookie.includes(testKey);
            JsCookie.remove(testKey);
        }
        return cookieEnabled;
    }
    get(key) {
        try {
            const value = JsCookie.get(key);
            if (!value) {
                return null;
            }
            try {
                return JSON.parse(value);
            }
            catch (e) {
                return value;
            }
        }
        catch (e) {
            return null;
        }
    }
    set(key, value) {
        const attributes = this.getAttributes();
        if (value === null || value === undefined || value === "") {
            JsCookie.remove(key, attributes);
            return null;
        }
        let parsedValue = "";
        switch (typeof value) {
            case "string":
                parsedValue = value;
                break;
            case "number":
            case "boolean":
                parsedValue = value.toString();
                break;
            default:
                parsedValue = JSON.stringify(value);
        }
        JsCookie.set(key, parsedValue, attributes);
        return value;
    }
    remove(key) {
        return JsCookie.remove(key, this.getAttributes());
    }
    getAttributes() {
        if (!this.attributes) {
            this.setAttributes();
        }
        return this.attributes;
    }
    setAttributes() {
        this.attributes = {
            expires: ONE_YEAR,
            path: "/",
            domain: this.getDomainAttribute(),
            sameSite: "Lax",
        };
    }
    getDomainAttribute() {
        if (this.domain) {
            return this.domain;
        }
        const parsedUrl = parseUrl(location === null || location === void 0 ? void 0 : location.href);
        if (!parsedUrl)
            return;
        const levels = getUrlLevels(parsedUrl);
        // Lookup the real top level one.
        for (let i = 0; i < levels.length; ++i) {
            const cname = "__tld__";
            const domain = levels[i];
            const opts = { domain: "." + domain };
            try {
                // cookie access throw an error if the library is run inside a sandboxed environment (e.g. sandboxed iframe)
                JsCookie.set(cname, "1", opts);
                if (JsCookie.get(cname)) {
                    JsCookie.remove(cname, opts);
                    return domain;
                }
            }
            catch (_) {
                return;
            }
        }
    }
}
function parseUrl(url) {
    try {
        return new URL(url);
    }
    catch (_a) {
        return;
    }
}
function getUrlLevels(url) {
    const host = url.hostname;
    const parts = host.split(".");
    const last = parts[parts.length - 1];
    const levels = [];
    // Ip address.
    if (parts.length === 4 && parseInt(last, 10) > 0) {
        return levels;
    }
    // Localhost.
    if (parts.length <= 1) {
        return levels;
    }
    // Create levels.
    for (let i = parts.length - 2; i >= 0; --i) {
        levels.push(parts.slice(i).join("."));
    }
    return levels;
}
//# sourceMappingURL=cookiesStore.js.map