import { MemoryStore } from "./memoryStore";
export class SessionStore {
    constructor(browser) {
        this.sessionStorage = null;
        this.memoryStore = null;
        if (this.isAvailable(browser)) {
            this.sessionStorage = browser.window().sessionStorage;
        }
        else {
            this.memoryStore = new MemoryStore();
        }
    }
    isAvailable(browser) {
        const testKey = "journify.io-test-browser-storage-key";
        try {
            browser
                .window()
                .sessionStorage.setItem(testKey, "journify.io-test-browser-storage-value");
            browser.window().sessionStorage.removeItem(testKey);
            return true;
        }
        catch (e) {
            return false;
        }
    }
    get(key) {
        const val = this.sessionStorage
            ? this.sessionStorage.getItem(key)
            : this.memoryStore.get(key);
        if (val) {
            try {
                return JSON.parse(val);
            }
            catch (e) {
                return JSON.parse(JSON.stringify(val));
            }
        }
        return null;
    }
    set(key, value) {
        try {
            const stringValue = JSON.stringify(value);
            this.sessionStorage
                ? this.sessionStorage.setItem(key, stringValue)
                : this.memoryStore.set(key, stringValue);
        }
        catch (e) {
            console.warn(`Unable to set ${key} in session storage, storage may be full.`);
            console.warn(e);
            return null;
        }
        return value;
    }
    remove(key) {
        try {
            this.sessionStorage
                ? this.sessionStorage.removeItem(key)
                : this.memoryStore.remove(key);
        }
        catch (e) {
            console.warn(`Unable to remove ${key} from session storage.`);
            console.warn(e);
        }
    }
}
//# sourceMappingURL=sessionStore.js.map