export class BrowserImpl {
    injectScript(url, opts) {
        var _a;
        const script = this.document().createElement("script");
        script.src = url;
        script.async = opts.async;
        script.onload = opts.onload;
        const doc = this.document();
        const scripts = doc.getElementsByTagName("script");
        if (scripts.length > 0) {
            (_a = scripts[0].parentNode) === null || _a === void 0 ? void 0 : _a.insertBefore(script, null);
        }
        else {
            doc.head.appendChild(script);
        }
    }
    scriptAlreadyInPage(scriptUrl) {
        return Array.from(this.document().scripts).some((script) => script.src.includes(scriptUrl));
    }
    isCurrentPageHttps() {
        var _a;
        return ((_a = this.location()) === null || _a === void 0 ? void 0 : _a.protocol) === "https:";
    }
    navigator() {
        return navigator;
    }
    document() {
        return document;
    }
    location() {
        return location;
    }
    window() {
        return window;
    }
    isOnline() {
        var _a;
        return ((_a = this.navigator()) === null || _a === void 0 ? void 0 : _a.onLine) || false;
    }
    canonicalPath() {
        var _a;
        const canon = this.canonical();
        if (!canon) {
            return (_a = this.location()) === null || _a === void 0 ? void 0 : _a.pathname;
        }
        const a = this.document().createElement("a");
        a.href = canon;
        return !a.pathname.startsWith("/") ? "/" + a.pathname : a.pathname;
    }
    canonicalUrl() {
        var _a, _b;
        const canon = this.canonical();
        if (canon) {
            return canon.includes("?") ? canon : `${canon}${(_a = this.location()) === null || _a === void 0 ? void 0 : _a.search}`;
        }
        const url = (_b = this.location()) === null || _b === void 0 ? void 0 : _b.href;
        const i = url.indexOf("#");
        return i === -1 ? url : url.slice(0, i);
    }
    utmCampaign(queryString, store) {
        const utm = {};
        const sParams = new URLSearchParams(queryString);
        let campaignFound = false;
        UTM_KEYS.forEach((k) => {
            const paramName = k[0];
            const param = this.getUtmParam(paramName, sParams, store);
            if (param) {
                const fieldName = k[1];
                campaignFound = true;
                utm[fieldName] = param;
            }
        });
        return campaignFound ? utm : undefined;
    }
    canonical() {
        const tag = this.document().querySelector("link[rel='canonical']");
        if (!tag) {
            return null;
        }
        return tag.getAttribute("href");
    }
    getUtmParam(paramName, sParams, store) {
        const value = sParams.get(paramName) || store.get(paramName);
        if (!value) {
            return null;
        }
        store.set(paramName, value);
        return value.toString();
    }
}
export const UTM_KEYS = [
    ["utm_id", "id"],
    ["utm_campaign", "name"],
    ["utm_source", "source"],
    ["utm_medium", "medium"],
    ["utm_term", "term"],
    ["utm_content", "content"],
];
//# sourceMappingURL=browser.js.map