var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import * as uuid from "uuid";
import { JournifyEventType } from "../domain/event";
import { LIB_VERSION } from "../generated/libVersion";
import { SESSION_ID_PERSISTENCE_KEY } from "../store/store";
import { SessionStore } from "../store/sessionStore";
export class EventFactoryImpl {
    constructor(stores, cookiesStore, browser, externalIdsSessionCache) {
        this.stores = stores;
        this.cookiesStore = cookiesStore;
        this.browser = browser;
        this.externalIdsSessionCache = externalIdsSessionCache;
    }
    setGroup(group) {
        this.group = group;
    }
    setUser(user) {
        this.user = user;
    }
    newIdentifyEvent() {
        var _a, _b, _c;
        const baseEvent = {
            type: JournifyEventType.IDENTIFY,
            userId: ((_a = this.user) === null || _a === void 0 ? void 0 : _a.getUserId()) || null,
            anonymousId: (_b = this.user) === null || _b === void 0 ? void 0 : _b.getAnonymousId(),
            traits: (_c = this.user) === null || _c === void 0 ? void 0 : _c.getTraits(),
        };
        return this.normalizeEvent(baseEvent);
    }
    newTrackEvent(eventName, properties, traits) {
        var _a, _b, _c;
        return __awaiter(this, void 0, void 0, function* () {
            yield this.appendTraitsFromProperties(traits, properties);
            const mergedTraits = Object.assign(Object.assign({}, (((_a = this.user) === null || _a === void 0 ? void 0 : _a.getTraits()) || {})), traits);
            const baseEvent = {
                type: JournifyEventType.TRACK,
                event: eventName,
                properties,
                userId: ((_b = this.user) === null || _b === void 0 ? void 0 : _b.getUserId()) || mergedTraits.userId || null,
                anonymousId: (_c = this.user) === null || _c === void 0 ? void 0 : _c.getAnonymousId(),
                traits: mergedTraits,
            };
            return this.normalizeEvent(baseEvent);
        });
    }
    newPageEvent(pageName, properties, traits) {
        var _a, _b, _c;
        return __awaiter(this, void 0, void 0, function* () {
            yield this.appendTraitsFromProperties(traits, properties);
            const mergedTraits = Object.assign(Object.assign({}, (((_a = this.user) === null || _a === void 0 ? void 0 : _a.getTraits()) || {})), traits);
            const baseEvent = {
                type: JournifyEventType.PAGE,
                name: pageName,
                properties,
                userId: ((_b = this.user) === null || _b === void 0 ? void 0 : _b.getUserId()) || mergedTraits.userId || null,
                anonymousId: (_c = this.user) === null || _c === void 0 ? void 0 : _c.getAnonymousId(),
                traits: mergedTraits,
            };
            return this.normalizeEvent(baseEvent);
        });
    }
    newGroupEvent() {
        var _a, _b, _c;
        const baseEvent = {
            type: JournifyEventType.GROUP,
            userId: ((_a = this.user) === null || _a === void 0 ? void 0 : _a.getUserId()) || null,
            anonymousId: (_b = this.user) === null || _b === void 0 ? void 0 : _b.getAnonymousId(),
            traits: (_c = this.group) === null || _c === void 0 ? void 0 : _c.getTraits(),
        };
        return this.normalizeEvent(baseEvent);
    }
    normalizeEvent(baseEvent) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const ctx = (baseEvent === null || baseEvent === void 0 ? void 0 : baseEvent.context) || {};
            ctx.userAgent = this.browser.navigator().userAgent;
            ctx.groupId = ((_a = this.group) === null || _a === void 0 ? void 0 : _a.getGroupId()) || null;
            ctx.page = {
                referrer: this.browser.document().referrer,
                search: this.browser.location().search,
                title: this.browser.document().title,
                path: this.browser.canonicalPath(),
                url: this.browser.canonicalUrl(),
            };
            // TODO: Refactor StoreGroup to choose which storage you want to use
            const campaign = this.browser.utmCampaign(this.browser.location().search, new SessionStore(this.browser));
            if (campaign) {
                ctx.campaign = campaign;
            }
            if (!ctx.locale) {
                const navigator = this.browser.navigator();
                ctx.locale = navigator
                    ? navigator.language || navigator["userLanguage"]
                    : undefined;
            }
            if (!ctx.library) {
                ctx.library = {
                    name: "@journifyio/js-sdk",
                    version: LIB_VERSION,
                };
            }
            const sessionId = this.stores.get(SESSION_ID_PERSISTENCE_KEY);
            if (sessionId) {
                ctx.session = {
                    id: sessionId,
                };
            }
            const normalizedEvent = Object.assign(Object.assign({}, baseEvent), { messageId: uuid.v4(), timestamp: new Date(), externalIds: this.getExternalIDs(), context: ctx });
            return normalizedEvent;
        });
    }
    appendTraitsFromProperties(traits, properties) {
        return __awaiter(this, void 0, void 0, function* () {
            if (properties === null || properties === void 0 ? void 0 : properties.hashed_email) {
                traits.hashed_email = properties.hashed_email;
            }
            if (properties === null || properties === void 0 ? void 0 : properties.email) {
                traits.email = properties.email;
            }
            if (properties === null || properties === void 0 ? void 0 : properties.hashed_phone) {
                traits.hashed_phone = properties.hashed_phone;
            }
            if (properties === null || properties === void 0 ? void 0 : properties.phone) {
                traits.phone = properties.phone;
            }
        });
    }
    getExternalIDs() {
        var _a, _b;
        const potentialExternalIds = Object.assign({ facebook_click_id: this.cookiesStore.get("_fbc"), pinterest_click_id: this.cookiesStore.get("_epik"), tiktok_click_id: this.cookiesStore.get("ttclid"), snapchat_click_id: this.cookiesStore.get("_scclid"), facebook_browser_id: this.cookiesStore.get("_fbp"), snapchat_scid: this.cookiesStore.get("_scid"), tiktok_ttp: this.cookiesStore.get("_ttp"), microsoft_click_id: this.cookiesStore.get("_uetmsclkid"), linkedin_click_id: this.cookiesStore.get("li_fat_id"), google_click_id: this.getGoogleClickId(), twitter_click_id: this.getTwitterClickId(), snapchat_advertiser_cookie_1: this.cookiesStore.get("snapchat_advertiser_cookie_1") ||
                ((_a = this.user) === null || _a === void 0 ? void 0 : _a.getAnonymousId()) }, this.externalIdsSessionCache.getExternalIds());
        const userExternalIds = (_b = this.user) === null || _b === void 0 ? void 0 : _b.getExternalIds();
        if (userExternalIds) {
            for (const key in userExternalIds) {
                potentialExternalIds[key] = userExternalIds[key];
            }
        }
        return Object.entries(potentialExternalIds).reduce((accumulator, [key, value]) => {
            if (value) {
                accumulator[key] = "" + value;
            }
            return accumulator;
        }, {});
    }
    getGoogleClickId() {
        const gclAwID = this.getGoogleClickIdFromGclCookie("_gcl_aw");
        if ((gclAwID === null || gclAwID === void 0 ? void 0 : gclAwID.length) > 0) {
            return gclAwID;
        }
        const gclDcId = this.getGoogleClickIdFromGclCookie("_gcl_dc");
        if ((gclDcId === null || gclDcId === void 0 ? void 0 : gclDcId.length) > 0) {
            return gclDcId;
        }
        const fpgClaw = this.getGoogleClickIdFromFPGCookie("FPGCLAW");
        if ((fpgClaw === null || fpgClaw === void 0 ? void 0 : fpgClaw.length) > 0) {
            return fpgClaw;
        }
        return this.getGoogleClickIdFromFPGCookie("FPGCLDC");
    }
    getGoogleClickIdFromGclCookie(cookieName) {
        const cookieValue = this.cookiesStore.get(cookieName);
        if (!cookieValue) {
            return null;
        }
        const split = cookieValue.split(".");
        if (split.length < 3) {
            return null;
        }
        return split.slice(2).join("");
    }
    getGoogleClickIdFromFPGCookie(cookieName) {
        const cookieValue = this.cookiesStore.get(cookieName);
        if (!cookieValue) {
            return null;
        }
        const startIndex = cookieValue.indexOf(".k");
        if (startIndex === -1) {
            return null;
        }
        const endIndex = cookieValue.indexOf("$", startIndex);
        if (endIndex === -1) {
            return null;
        }
        return cookieValue.substring(startIndex + 2, endIndex);
    }
    getTwitterClickId() {
        let cookie = this.cookiesStore.get("_twclid");
        if (!cookie) {
            return cookie;
        }
        try {
            cookie = decodeURI(cookie);
            const obj = JSON.parse(cookie);
            if (obj === null || obj === void 0 ? void 0 : obj.twclid) {
                return obj.twclid;
            }
            return cookie;
        }
        catch (e) {
            return cookie;
        }
    }
}
//# sourceMappingURL=eventFactory.js.map