var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { toSettingsObject } from "../lib/settings";
import { getStoredIdentify } from "../lib/identify";
import { hashPII } from "../lib/hashPII";
import { matchFilters } from "../lib/filters";
// Event Paramaters https://help.ads.microsoft.com/apex/index/3/en/60123
export class BingAdsTag {
    constructor(deps) {
        this.name = "bing_ads_tag";
        this.browser = deps.browser;
        this.user = deps.user;
        this.testingMode = deps.testingWriteKey;
        this.enableHashing = deps.enableHashing;
        this.fieldsMapper = deps.fieldMapperFactory.newFieldMapper(deps.sync.field_mappings);
        this.eventMapper = deps.eventMapperFactory.newEventMapper(deps.sync.event_mappings);
        this.settings = {};
        this.logger = deps.logger;
        this.init(deps.sync);
    }
    updateSettings(sync) {
        this.init(sync);
    }
    identify(ctx) {
        return __awaiter(this, void 0, void 0, function* () {
            const newEvent = ctx.getEvent();
            const storedEvent = getStoredIdentify(this.user);
            let traits = Object.assign(Object.assign({}, (storedEvent.traits || {})), (newEvent.traits || {}));
            if (this.enableHashing) {
                traits = yield hashPII(traits);
            }
            const pid = {};
            if (traits.email) {
                pid["em"] = traits.email;
            }
            if (traits.phone) {
                pid["ph"] = traits.phone;
            }
            this.browser.window().uetq.push("set", { pid });
            return ctx;
        });
    }
    track(ctx) {
        var _a;
        const event = ctx.getEvent();
        const eventMapping = this.eventMapper.getEventMapping(event);
        if (!eventMapping || !matchFilters(event, eventMapping === null || eventMapping === void 0 ? void 0 : eventMapping.filters)) {
            return ctx;
        }
        const mappedProperties = this.fieldsMapper.mapEvent(event);
        (_a = this.browser
            .window()) === null || _a === void 0 ? void 0 : _a.uetq.push("event", eventMapping.pixelEventName, mappedProperties);
        return ctx;
    }
    page(ctx) {
        var _a;
        (_a = this.browser.window().uetq) === null || _a === void 0 ? void 0 : _a.push("pageLoad");
        return ctx;
    }
    group(ctx) {
        return ctx;
    }
    init(sync) {
        this.settings = toSettingsObject(sync.settings);
        if (!this.settings.bing_ads_tag_id) {
            this.logger.log("Bing Ads Tag is not injected because the pixel id is not set.");
            return;
        }
        this.loadScript(this.settings.bing_ads_tag_id);
    }
    loadScript(bingAdsTagId) {
        const localWindow = this.browser.window();
        if (localWindow.uetq) {
            return;
        }
        const scriptUrl = `https://bat.bing.com/bat.js`;
        const async = true;
        const onload = () => {
            localWindow.uetq = localWindow.uetq || [];
            const setup = {
                ti: bingAdsTagId,
                q: localWindow.uetq,
            };
            localWindow.uetq = new localWindow.UET(setup);
        };
        const opts = {
            async: async,
            onload: onload,
        };
        this.browser.injectScript(scriptUrl, opts);
    }
}
//# sourceMappingURL=bing_ads_tag.js.map