var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { toSettingsObject } from "../lib/settings";
import _ from "lodash";
import { hashPII } from "../lib/hashPII";
export class CleverTapPlugin {
    constructor(deps) {
        this.name = "celvertap";
        this.user = deps.user;
        this.sdk = deps.externalSDK;
        this.enableHashing = deps.enableHashing;
        this.init(deps.sync);
    }
    identify(ctx) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            let traits = ((_a = ctx.getEvent()) === null || _a === void 0 ? void 0 : _a.traits) || {};
            if (this.user) {
                traits = Object.assign(Object.assign({}, this.user.getTraits()), traits);
            }
            if (this.enableHashing) {
                traits = yield hashPII(traits);
            }
            const siteData = {
                Identity: (_b = this.user) === null || _b === void 0 ? void 0 : _b.getUserId(),
            };
            for (const [key, value] of Object.entries(traits)) {
                switch (key) {
                    case "birthday":
                        siteData.DOB = value;
                        break;
                    case "avatar":
                        siteData.Photo = value;
                        break;
                    case "gender":
                        siteData.Gender = value.charAt(0).toUpperCase();
                        break;
                    case "email":
                        siteData.Email = value;
                        break;
                    case "phone":
                        siteData.Phone = value;
                        break;
                    case "name":
                        siteData.Name = value;
                        break;
                    default:
                        siteData[key] = value;
                }
            }
            this.sdk.pushProfileData({ Site: siteData });
            return ctx;
        });
    }
    page(ctx) {
        const event = ctx.getEvent();
        const eventPage = event.context.page;
        const cleverTapEventProps = Object.assign({ Referrer: eventPage.referrer, Search: eventPage.search, Title: eventPage.title, Path: eventPage.path, URL: eventPage.url, UserAgent: event.context.userAgent }, event.properties);
        this.sdk.pushEvent("Web Page Viewed", cleverTapEventProps);
        return ctx;
    }
    track(ctx) {
        const event = ctx.getEvent();
        let eventName = event.event;
        if (eventName === "Order Completed") {
            eventName = "Charged";
        }
        const eventProps = event.properties;
        for (const [key, value] of Object.entries(eventProps)) {
            if (_.isObject(value) || _.isArray(value)) {
                eventProps[key] = JSON.stringify(value);
            }
        }
        this.sdk.pushEvent(eventName, event.properties);
        return ctx;
    }
    updateSettings(sync) {
        this.init(sync);
    }
    group(ctx) {
        return ctx;
    }
    init(sync) {
        var _a;
        const settings = toSettingsObject(sync.settings);
        this.sdk.init(settings.account_id, (_a = settings.region) === null || _a === void 0 ? void 0 : _a.trim());
    }
}
//# sourceMappingURL=cleverTapPlugin.js.map