export class CleverTapWrapperImpl {
    constructor(browser, logger, testingWriteKey) {
        this.scriptInitialized = false;
        this.browser = browser;
        this.logger = logger;
        this.testingMode = testingWriteKey;
    }
    init(accountId, region) {
        this.scriptInitialized = false;
        this.initScript();
        if (this.testingMode) {
            this.logger.log(`clevertap.account.push is called with: accountID: ${accountId}, region: ${region}`);
        }
        else {
            const clevertap = this.getCleverTapObject();
            clevertap.account.push({ id: accountId }, region);
        }
    }
    pushProfileData(profileData) {
        this.initScript();
        if (this.testingMode) {
            this.logger.log("Will call clevertap.onUserLogin.push with the following param:", profileData);
            return;
        }
        const clevertap = this.getCleverTapObject();
        clevertap.onUserLogin.push(profileData);
    }
    pushEvent(evtName, ...evtNameOrData) {
        this.initScript();
        if (this.testingMode) {
            this.logger.log("Will call clevertap.event.push with the following params in order:", [evtName, ...evtNameOrData]);
            return;
        }
        const clevertap = this.getCleverTapObject();
        clevertap.event.push(evtName, ...evtNameOrData);
    }
    initScript() {
        if (this.scriptInitialized) {
            return;
        }
        else if (this.testingMode) {
            this.logger.log("CleverTap is detected but script is not injected because you are using a testing write key.");
            this.scriptInitialized = true;
            return;
        }
        const win = this.browser.window();
        win.clevertap = {
            account: [],
            event: [],
            profile: [],
            onUserLogin: [],
            notifications: [],
            privacy: [],
        };
        const isHttps = this.browser.isCurrentPageHttps();
        const scriptHost = isHttps
            ? "https://d2r1yp2w7bby2u.cloudfront.net"
            : "http://static.clevertap.com";
        const scriptUrl = scriptHost + "/js/clevertap.min.js";
        this.browser.injectScript(scriptUrl, { async: true });
        this.scriptInitialized = true;
    }
    getCleverTapObject() {
        return this.browser.window().clevertap;
    }
}
//# sourceMappingURL=cleverTapWrapper.js.map