var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/* eslint-disable  @typescript-eslint/no-explicit-any */
import { JournifyEventType } from "../../../domain/event";
import { getStoredIdentify } from "../lib/identify";
import { toSettingsObject } from "../lib/settings";
import { hashPII } from "../lib/hashPII";
export class GA4Pixel {
    constructor(deps) {
        this.name = "ga4_pixel";
        this.user = deps.user;
        this.browser = deps.browser;
        this.testingMode = deps.testingWriteKey;
        this.logger = deps.logger;
        this.enableHashing = deps.enableHashing;
        this.init(deps.sync);
    }
    // implement the interface
    identify(ctx) {
        return __awaiter(this, void 0, void 0, function* () {
            const newEvent = ctx.getEvent();
            const storedEvent = getStoredIdentify(this.user);
            const event = {
                type: JournifyEventType.IDENTIFY,
                userId: newEvent.userId || storedEvent.userId,
                anonymousId: newEvent.anonymousId || storedEvent.anonymousId,
                traits: Object.assign(Object.assign({}, (storedEvent.traits || {})), (newEvent.traits || {})),
            };
            if (this.enableHashing) {
                event.traits = yield hashPII(event.traits);
            }
            if (Object.keys(event.traits).length > 0) {
                window.gtag("set", "user_properties", event.traits);
            }
            this.browser.window().gtag("config", this.settings.mesurement_id, {
                user_id: event.userId,
            });
            return ctx;
        });
    }
    track(ctx) {
        return this.trackPixelEvent(ctx);
    }
    page(ctx) {
        return this.trackPixelEvent(ctx);
    }
    group(ctx) {
        return this.trackPixelEvent(ctx);
    }
    updateSettings(sync) {
        this.init(sync);
    }
    trackPixelEvent(ctx) {
        const event = ctx.getEvent();
        this.callPixelHelper(event.type, event.event, event.properties);
        return ctx;
    }
    callPixelHelper(type, event, args) {
        switch (type) {
            case JournifyEventType.PAGE:
                if (this.testingMode) {
                    return this.logger.log("Will call window.gtag with the following params in order:", ["event", "page_view", args]);
                }
                this.browser.window().gtag("event", "page_view", args);
                break;
            case JournifyEventType.GROUP:
                if (this.testingMode) {
                    return this.logger.log("Will call window.gtag with the following params in order:", ["event", "join_group", args]);
                }
                this.browser.window().gtag("event", "join_group", args);
                break;
            default:
                if (this.testingMode) {
                    return this.logger.log("Will call window.gtag with the following params in order:", ["event", event, args]);
                }
                this.browser.window().gtag("event", event, args);
        }
    }
    init(sync) {
        this.settings = toSettingsObject(sync.settings);
        this.loadScript(this.settings.mesurement_id);
    }
    loadScript(measurementID) {
        const localWindow = this.browser.window();
        localWindow.JDataLayer = localWindow.JDataLayer || [];
        // Using a custom data layer to avoid conflicts with gtags
        // Docs: https://developers.google.com/tag-platform/tag-manager/datalayer?hl=en#rename_the_data_layer
        localWindow.gtag = function gtag() {
            // This is very important to use the arguments object here, and it's not possible to use rest parameters
            // Pushing to our custom data layer
            // eslint-disable-next-line prefer-rest-params
            localWindow.JDataLayer.push(arguments);
        };
        localWindow.gtag("js", new Date());
        localWindow.gtag("config", measurementID);
        // Load the gtag script with l paramater to set the new data layer name
        this.browser.injectScript(`https://www.googletagmanager.com/gtag/js?id=${measurementID}&l=JDataLayer`, { async: true });
    }
}
//# sourceMappingURL=ga4PPixel.js.map