var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { hashPII } from "../lib/hashPII";
const DEFAULT_API_HOST = "https://t.journify.io";
// this is used to limit the size of the request body to prevent issues with large payloads
const MAX_REQUEST_BODY_SIZE_KB = 16; // 16KB
export class JournifyioPlugin {
    constructor(sdkConfig, sentry) {
        this.name = "journifyio";
        this.identify = this.post;
        this.track = this.post;
        this.page = this.post;
        this.group = this.post;
        this.sdkSettings = sdkConfig;
        this.sentry = sentry;
    }
    updateSettings(settings) {
        this.sdkSettings = settings;
    }
    post(ctx) {
        var _a, _b, _c, _d, _e, _f;
        return __awaiter(this, void 0, void 0, function* () {
            const apiHost = (_a = this.sdkSettings.apiHost) !== null && _a !== void 0 ? _a : DEFAULT_API_HOST;
            const event = ctx.getEvent();
            if (((_c = (_b = this.sdkSettings) === null || _b === void 0 ? void 0 : _b.options) === null || _c === void 0 ? void 0 : _c.enableHashing) === true) {
                event.traits = yield hashPII(event.traits);
                event.externalIds = yield hashPII(event.externalIds);
            }
            if (((_d = event.traits) === null || _d === void 0 ? void 0 : _d.hashed_email) && !event.traits.email) {
                event.traits.email = event.traits.hashed_email;
                delete event.traits.hashed_email;
            }
            if (((_e = event.traits) === null || _e === void 0 ? void 0 : _e.hashed_phone) && !event.traits.phone) {
                event.traits.phone = event.traits.hashed_phone;
                delete event.traits.hashed_phone;
            }
            const eventUrl = `${apiHost}/v1/${(_f = event.type) === null || _f === void 0 ? void 0 : _f.charAt(0)}`;
            const requestBody = Object.assign(Object.assign({}, event), { writeKey: this.sdkSettings.writeKey });
            const requestBodyStr = JSON.stringify(requestBody);
            const requestBodyBytes = new TextEncoder().encode(requestBodyStr).length;
            if (requestBodyBytes > MAX_REQUEST_BODY_SIZE_KB * 1024) {
                throw new Error(`Your event is ignored because the request body exceeds ${MAX_REQUEST_BODY_SIZE_KB}Kb limit.`);
            }
            this.sentry.setTag("eventUrl", eventUrl);
            this.sentry.setTag("writeKey", requestBody.writeKey);
            const response = yield fetch(eventUrl, {
                method: "POST",
                headers: {
                    "Content-Type": "application/json",
                },
                body: requestBodyStr,
                keepalive: true,
            });
            if (!response.ok) {
                this.sentry.setResponse({
                    url: eventUrl,
                    headers: response.headers,
                    status: response.status,
                    body: yield response.text(),
                });
                // Only throw error if it's 5xx (server errors) error to retry
                if (response.status >= 500) {
                    throw new Error(`POST request to ${eventUrl} returned ${response.status} HTTP status`);
                }
            }
            return ctx;
        });
    }
}
//# sourceMappingURL=journifyio.js.map