import { toSettingsObject } from "../lib/settings";
import { matchFilters } from "../lib/filters";
import { toInt } from "../lib/tranformations";
const LINKEDIN_INSIGHT_TAG_SCRIPT_URL = "https://snap.licdn.com/li.lms-analytics/insight.min.js";
export class LinkedinAdsInsightTag {
    constructor(deps) {
        this.name = "linkedin_ads_insight_tag";
        this.settings = {};
        this.browser = deps.browser;
        this.fieldsMapper = deps.fieldMapperFactory.newFieldMapper(deps.sync.field_mappings);
        this.testingMode = deps.testingWriteKey;
        this.eventMapper = deps.eventMapperFactory.newEventMapper(deps.sync.event_mappings);
        this.logger = deps.logger;
        this.init(deps.sync);
    }
    group(ctx) {
        return this.trackInsightTagEvent(ctx);
    }
    identify(ctx) {
        return this.trackInsightTagEvent(ctx);
    }
    page(ctx) {
        return this.trackInsightTagEvent(ctx);
    }
    track(ctx) {
        return this.trackInsightTagEvent(ctx);
    }
    updateSettings(sync) {
        this.init(sync);
    }
    init(sync) {
        this.settings = toSettingsObject(sync.settings);
        if (this.testingMode) {
            this.logger.log(`Linkedin Insight Tag ${this.settings.linkedin_partner_id} is detected, but script is not injected because you are using a testing write key.`);
        }
        else {
            this.loadTagScript();
        }
    }
    loadTagScript() {
        const localWindow = this.browser.window();
        localWindow._linkedin_data_partner_ids =
            window._linkedin_data_partner_ids || [];
        localWindow._linkedin_data_partner_ids.push(this.settings.linkedin_partner_id);
        const linktrk = localWindow.lintrk;
        if (!linktrk) {
            localWindow.lintrk = function (a, b) {
                localWindow.lintrk.q.push([a, b]);
            };
            localWindow.lintrk.q = [];
        }
        this.browser.injectScript(LINKEDIN_INSIGHT_TAG_SCRIPT_URL, { async: true });
    }
    trackInsightTagEvent(ctx) {
        const event = ctx.getEvent();
        const eventMapping = this.eventMapper.getEventMapping(event);
        if (!eventMapping || !matchFilters(event, eventMapping === null || eventMapping === void 0 ? void 0 : eventMapping.filters)) {
            return ctx;
        }
        const properties = this.fieldsMapper.mapEvent(event, {}, { ignoreUnmappedProperties: true });
        this.lintrk("track", Object.assign({ conversion_id: toInt(eventMapping.pixelEventName) }, properties));
        return ctx;
    }
    lintrk(...args) {
        if (this.testingMode) {
            this.logger.log("Will call window.lintrk with the following params in order:", args);
            return;
        }
        this.browser.window().lintrk(...args);
    }
}
//# sourceMappingURL=linkedinAdsInsightTag.js.map