import { EventMapperImpl, } from "../lib/mapping";
import { toSettingsObject } from "../lib/settings";
import { getStoredIdentify } from "../lib/identify";
import { JournifyEventType } from "../../../domain/event";
import { toLowerCase, trim } from "../lib/tranformations";
import { matchFilters } from "../lib/filters";
const PINTEREST_PIXEL_SCRIPT_URL = "https://s.pinimg.com/ct/core.js";
export class PinterestTag {
    constructor(deps) {
        this.name = "pinterest_tag";
        this.user = deps.user;
        this.eventMapper = deps.eventMapperFactory.newEventMapper(deps.sync.event_mappings);
        this.browser = deps.browser;
        this.fieldMapperFactory = deps.fieldMapperFactory;
        this.testingMode = deps.testingWriteKey;
        this.logger = deps.logger;
        this.init(deps.sync);
    }
    identify(ctx) {
        const newEvent = ctx.getEvent();
        const storedEvent = getStoredIdentify(this.user);
        const event = {
            type: JournifyEventType.IDENTIFY,
            userId: newEvent.userId || storedEvent.userId,
            anonymousId: newEvent.anonymousId || storedEvent.anonymousId,
            traits: Object.assign(Object.assign({}, (storedEvent.traits || {})), (newEvent.traits || {})),
        };
        this.initTag(event);
        return this.trackTagEvent(ctx);
    }
    page(ctx) {
        return this.trackTagEvent(ctx);
    }
    track(ctx) {
        return this.trackTagEvent(ctx);
    }
    group(ctx) {
        return this.trackTagEvent(ctx);
    }
    updateSettings(sync) {
        this.init(sync);
    }
    init(sync) {
        this.fieldsMapper = this.fieldMapperFactory.newFieldMapper(sync.field_mappings);
        this.eventMapper = new EventMapperImpl(sync.event_mappings);
        this.settings = toSettingsObject(sync.settings);
        if (this.testingMode) {
            this.logger.log(`Pinterest Tag ${this.settings.tag_id} is detected, but script is not injected because you are using a testing write key.`);
        }
        else {
            this.loadScript();
        }
        const event = getStoredIdentify(this.user);
        this.initTag(event);
    }
    loadScript() {
        const localWindow = this.browser.window();
        if (localWindow.pintrk) {
            return;
        }
        // eslint-disable-next-line @typescript-eslint/ban-ts-comment
        // @ts-ignore
        localWindow.pintrk = function () {
            // eslint-disable-next-line prefer-rest-params
            localWindow.pintrk.queue.push(Array.prototype.slice.call(arguments));
        };
        const localDocument = this.browser.document();
        const n = localWindow.pintrk;
        (n.queue = []), (n.version = "3.0");
        const t = localDocument.createElement("script");
        (t.async = !0), (t.src = PINTEREST_PIXEL_SCRIPT_URL);
        const r = localDocument.getElementsByTagName("script")[0];
        r.parentNode.insertBefore(t, r);
    }
    initTag(identifyEvent) {
        const transformationsMap = {
            em: [trim, toLowerCase],
        };
        const userData = this.fieldsMapper.mapEvent(identifyEvent, transformationsMap, { ignoreUnmappedProperties: true });
        this.callPintrk("load", this.settings.tag_id, userData);
    }
    callPintrk(trackingType, ...args) {
        if (this.testingMode) {
            this.logger.log("Will call window.pintrk with the following params in order:", [trackingType, ...args]);
        }
        else {
            this.browser.window().pintrk(trackingType, ...args);
        }
    }
    trackTagEvent(ctx) {
        const event = ctx.getEvent();
        const eventMapping = this.eventMapper.getEventMapping(event);
        if (!eventMapping || !matchFilters(event, eventMapping === null || eventMapping === void 0 ? void 0 : eventMapping.filters)) {
            return ctx;
        }
        const mappedProperties = this.fieldsMapper.mapEvent(event, {}, { ignoreUnmappedProperties: true });
        const eventName = (eventMapping === null || eventMapping === void 0 ? void 0 : eventMapping.pixelEventName) || event.event;
        if (!eventMapping) {
            return ctx;
        }
        this.callPintrk("track", eventName, mappedProperties);
        return ctx;
    }
}
//# sourceMappingURL=pinterestTag.js.map