import { matchFilters } from "../lib/filters";
import { EventMapperImpl, } from "../lib/mapping";
import { toSettingsObject } from "../lib/settings";
const X_PIXEL_SCRIPT_URL = "https://static.ads-twitter.com/uwt.js";
export class XPixel {
    constructor(deps) {
        this.name = "x_pixel";
        this.eventMapper = deps.eventMapperFactory.newEventMapper(deps.sync.event_mappings);
        this.browser = deps.browser;
        this.fieldMapperFactory = deps.fieldMapperFactory;
        this.testingMode = deps.testingWriteKey;
        this.logger = deps.logger;
        this.init(deps.sync);
    }
    identify(ctx) {
        return this.trackPixelEvent(ctx);
    }
    track(ctx) {
        return this.trackPixelEvent(ctx);
    }
    group(ctx) {
        return this.trackPixelEvent(ctx);
    }
    updateSettings(sync) {
        this.init(sync);
    }
    page(ctx) {
        return this.trackPixelEvent(ctx);
    }
    trackPixelEvent(ctx) {
        const event = ctx.getEvent();
        const eventMapping = this.eventMapper.getEventMapping(event);
        if (!eventMapping || !matchFilters(event, eventMapping === null || eventMapping === void 0 ? void 0 : eventMapping.filters)) {
            return ctx;
        }
        const mappedProperties = this.fieldsMapper.mapEvent(event, {}, { ignoreUnmappedProperties: true });
        this.callPixelHelper("event", `tw-${this.settings.x_pixel_id}-${eventMapping.pixelEventName}`, mappedProperties);
        return ctx;
    }
    init(sync) {
        this.fieldsMapper = this.fieldMapperFactory.newFieldMapper(sync.field_mappings);
        this.eventMapper = new EventMapperImpl(sync.event_mappings);
        this.settings = toSettingsObject(sync.settings);
        if (this.testingMode) {
            this.logger.log(`X Pixel ${this.settings.x_pixel_id} is detected, but script is not injected because you are using a testing write key.`);
        }
        else {
            this.loadScript();
        }
        this.initPixel();
    }
    initPixel() {
        this.callPixelHelper("config", this.settings.x_pixel_id);
    }
    callPixelHelper(trackingType, ...args) {
        if (this.testingMode) {
            this.logger.log("Will call window.twq with the following params in order:", [trackingType, ...args]);
        }
        else {
            this.browser.window().twq(trackingType, ...args);
        }
    }
    /* eslint-disable */
    loadScript() {
        const localWindow = this.browser.window();
        if (localWindow.twq)
            return;
        const localDocument = this.browser.document();
        let s, u, a;
        s = localWindow.twq = function () {
            s.exe ? s.exe.apply(s, arguments) : s.queue.push(arguments);
        };
        s.version = "1.1";
        s.queue = [];
        u = localDocument.createElement("script");
        u.async = !0;
        u.src = X_PIXEL_SCRIPT_URL;
        a = localDocument.getElementsByTagName("script")[0];
        a.parentNode.insertBefore(u, a);
    }
}
//# sourceMappingURL=xPixel.js.map