var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { ContextFactoryImpl } from "./context";
import { ON_OPERATION_DELAY_FINISH, } from "../lib/priorityQueue";
import { EmitterImpl } from "./emitter";
const FLUSH_EVENT_NAME = "flush";
export class EventQueueImpl extends EmitterImpl {
    constructor(plugins, pQueue, browser, sentry, contextFactory = new ContextFactoryImpl()) {
        super();
        this.plugins = [];
        this.flushing = false;
        this.plugins = plugins;
        this.pQueue = pQueue;
        this.pQueue.on(ON_OPERATION_DELAY_FINISH, () => __awaiter(this, void 0, void 0, function* () {
            try {
                yield this.flush();
            }
            catch (error) {
                this.sentry.captureException(error);
                console.error(error);
            }
        }));
        this.browser = browser;
        this.sentry = sentry;
        this.contextFactory = contextFactory;
    }
    push(ctx) {
        const updatedContextEvents = this.plugins.map((p) => {
            const newId = generateNewIDWithPlugin(ctx.getId(), p.name);
            const newCtx = this.contextFactory.newContext(ctx.getEvent(), newId);
            return newCtx;
        });
        this.pQueue.push(...updatedContextEvents);
        return updatedContextEvents;
    }
    deliver(ctx) {
        return __awaiter(this, void 0, void 0, function* () {
            const updatedContexts = this.push(ctx);
            for (const c of updatedContexts) {
                this.subscribeToDelivery(c);
                this.flush();
            }
            return ctx;
        });
    }
    subscribeToDelivery(sentCtx) {
        this.on(FLUSH_EVENT_NAME, (flushedCtx, delivered) => {
            var _a;
            if (!flushedCtx.isSame(sentCtx) || delivered) {
                return;
            }
            const failureReason = (_a = flushedCtx.getFailedDelivery()) === null || _a === void 0 ? void 0 : _a.reason;
            this.sentry.captureException(failureReason);
            console.error(failureReason);
        });
    }
    flush() {
        return __awaiter(this, void 0, void 0, function* () {
            while (this.flushing)
                ;
            this.flushing = true;
            if (this.pQueue.isEmpty() || !this.browser.isOnline()) {
                this.flushing = false;
                return;
            }
            const ctxToDeliver = this.pQueue.pop();
            this.flushing = false;
            if (!ctxToDeliver) {
                return;
            }
            yield new Promise((resolve, reject) => {
                setTimeout(() => {
                    const plugin = this.plugins.find((p) => ctxToDeliver.getId().includes(p.name));
                    if (plugin) {
                        this.attempt(plugin, ctxToDeliver).then(resolve).catch(reject);
                    }
                    else {
                        this.sentry.captureMessage("unable to find a plugin for given event", null, {
                            data: {
                                event: ctxToDeliver,
                            },
                        });
                    }
                }, 0);
            });
        });
    }
    attempt(plugin, ctxToDeliver) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const deliveredCtx = yield this.runPlugin(ctxToDeliver, plugin);
                this.emit(FLUSH_EVENT_NAME, deliveredCtx, true);
            }
            catch (err) {
                this.handleFlushError(ctxToDeliver, plugin, err);
            }
        });
    }
    runPlugin(ctxToDeliver, plugin) {
        return __awaiter(this, void 0, void 0, function* () {
            const event = ctxToDeliver.getEvent();
            if (!plugin || !plugin[event.type]) {
                return ctxToDeliver;
            }
            const hook = plugin[event.type];
            return hook.apply(plugin, [ctxToDeliver]);
        });
    }
    handleFlushError(ctxToDeliver, plugin, err) {
        const retryAccepted = this.pQueue.pushWithBackoff(ctxToDeliver);
        if (!retryAccepted) {
            ctxToDeliver.setFailedDelivery({ reason: err });
            this.emit(FLUSH_EVENT_NAME, ctxToDeliver, false);
        }
    }
}
function generateNewIDWithPlugin(id, plugin) {
    return `${id}/${plugin}`;
}
//# sourceMappingURL=queue.js.map